@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/extended.c,v 1.32.2.1 2005/01/20 18:03:50 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/extended.c,v 1.39 2006/02/14 22:18:12 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2005 The OpenLDAP Foundation.
+ * Copyright 1998-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -61,7 +61,7 @@ ldap_extended_operation(
 
 	assert( ld != NULL );
 	assert( LDAP_VALID( ld ) );
-	assert( reqoid != NULL || *reqoid == '\0' );
+	assert( reqoid != NULL && *reqoid != '\0' );
 	assert( msgidp != NULL );
 
 	/* must be version 3 (or greater) */
@@ -131,8 +131,7 @@ ldap_extended_operation_s(
 
 	assert( ld != NULL );
 	assert( LDAP_VALID( ld ) );
-	assert( reqoid != NULL || *reqoid == '\0' );
-	assert( retoidp != NULL || retdatap != NULL );
+	assert( reqoid != NULL && *reqoid != '\0' );
 
     rc = ldap_extended_operation( ld, reqoid, reqdata,
 		sctrls, cctrls, &msgid );
@@ -141,7 +140,7 @@ ldap_extended_operation_s(
         return( rc );
 	}
  
-    if ( ldap_result( ld, msgid, 1, (struct timeval *) NULL, &res ) == -1 ) {
+    if ( ldap_result( ld, msgid, LDAP_MSG_ALL, (struct timeval *) NULL, &res ) == -1 ) {
         return( ld->ld_errno );
 	}
 
@@ -211,7 +210,7 @@ ldap_parse_extended_result (
 		return ld->ld_errno;
 	}
 
-	rc = ber_scanf( ber, "{iaa" /*}*/, &errcode,
+	rc = ber_scanf( ber, "{eAA" /*}*/, &errcode,
 		&ld->ld_matched, &ld->ld_error );
 
 	if( rc == LBER_ERROR ) {
@@ -244,6 +243,8 @@ ldap_parse_extended_result (
 			return ld->ld_errno;
 		}
 
+		assert( resoid[ 0 ] != '\0' );
+
 		tag = ber_peek_tag( ber, &len );
 	}
 
@@ -351,6 +352,8 @@ ldap_parse_intermediate (
 			return ld->ld_errno;
 		}
 
+		assert( resoid[ 0 ] != '\0' );
+
 		tag = ber_peek_tag( ber, &len );
 	}
 
