@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/getdn.c,v 1.124 2004/09/04 02:54:29 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/getdn.c,v 1.130 2006/01/12 05:07:42 hyc Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -358,7 +358,7 @@ ldap_dn_normalize( LDAP_CONST char *dnin,
 
 	Debug( LDAP_DEBUG_TRACE, "ldap_dn_normalize\n", 0, 0, 0 );
 
-	assert( dnout );
+	assert( dnout != NULL );
 
 	*dnout = NULL;
 
@@ -438,10 +438,9 @@ ldap_dn_normalize( LDAP_CONST char *dnin,
  * as "\=", but it is treated as a regular char, i.e. it can also 
  * appear as '='.
  *
- * As such, we currently choose to allow reading unescaped '=',
- * but we always produce escaped '\3D'; this may change in the
- * future, if compatibility issues do not arise */
-#ifdef LDAP_DEVEL
+ * As such, in 2.2 we used to allow reading unescaped '=',
+ * but we always produced escaped '\3D'; this changes 
+ * since 2.3, if compatibility issues do not arise */
 #define LDAP_DN_NE(c) \
 	( LDAP_DN_RDN_SEP_V2(c) || LDAP_DN_AVA_SEP(c) \
 	  || LDAP_DN_QUOTES(c) \
@@ -451,17 +450,7 @@ ldap_dn_normalize( LDAP_CONST char *dnin,
 	  || LDAP_DN_AVA_EQUALS(c) \
 	  || LDAP_DN_ASCII_SPACE(c) || LDAP_DN_OCTOTHORPE(c) )
 #define LDAP_DN_SHOULDESCAPE(c)		( LDAP_DN_AVA_EQUALS(c) )
-#else /* ! LDAP_DEVEL */
-#define LDAP_DN_NE(c) \
-	( LDAP_DN_RDN_SEP_V2(c) || LDAP_DN_AVA_SEP(c) \
-	  || LDAP_DN_AVA_EQUALS(c) || LDAP_DN_QUOTES(c) \
-	  || (c) == '<' || (c) == '>' )
-#define LDAP_DN_MAYESCAPE(c) \
-	( LDAP_DN_ESCAPE(c) || LDAP_DN_NE(c) \
-	  || LDAP_DN_ASCII_SPACE(c) || LDAP_DN_OCTOTHORPE(c) )
-#define LDAP_DN_SHOULDESCAPE(c)		( 0 )
-#endif /* ! LDAP_DEVEL */
-	
+
 #define LDAP_DN_NEEDESCAPE(c) \
 	( LDAP_DN_ESCAPE(c) || LDAP_DN_NE(c) )
 #define LDAP_DN_NEEDESCAPE_LEAD(c) 	LDAP_DN_MAYESCAPE(c)
@@ -568,14 +557,14 @@ ldap_dn_normalize( LDAP_CONST char *dnin,
  * LDAPAVA helpers (will become part of the API for operations 
  * on structural representations of DNs).
  */
-LDAPAVA *
+static LDAPAVA *
 ldapava_new( const struct berval *attr, const struct berval *val, 
 		unsigned flags, void *ctx )
 {
 	LDAPAVA *ava;
 
-	assert( attr );
-	assert( val );
+	assert( attr != NULL );
+	assert( val != NULL );
 
 	ava = LDAP_MALLOCX( sizeof( LDAPAVA ) + attr->bv_len + 1, ctx );
 
@@ -597,7 +586,7 @@ ldapava_new( const struct berval *attr, const struct berval *val,
 void
 ldapava_free( LDAPAVA *ava, void *ctx )
 {
-	assert( ava );
+	assert( ava != NULL );
 
 #if 0
 	/* ava's private must be freed by caller
@@ -684,7 +673,7 @@ ldap_str2dn( LDAP_CONST char *str, LDAPDN *dn, unsigned flags )
 {
 	struct berval	bv;
 
-	assert( str );
+	assert( str != NULL );
 
 	bv.bv_len = strlen( str );
 	bv.bv_val = (char *) str;
@@ -711,15 +700,15 @@ ldap_bv2dn_x( struct berval *bvin, LDAPDN *dn, unsigned flags, void *ctx )
 	char		*str, *end;
 	struct berval	bvtmp, *bv = &bvtmp;
 	
-	assert( bvin );
-	assert( bvin->bv_val );
-	assert( dn );
+	assert( bvin != NULL );
+	assert( bvin->bv_val != NULL );
+	assert( dn != NULL );
 
 	*bv = *bvin;
 	str = bv->bv_val;
 	end = str + bv->bv_len;
 
-	Debug( LDAP_DEBUG_TRACE, "=> ldap_bv2dn(%s,%u)\n", str, flags, 0 );
+	Debug( LDAP_DEBUG_ARGS, "=> ldap_bv2dn(%s,%u)\n", str, flags, 0 );
 
 	*dn = NULL;
 
@@ -901,8 +890,8 @@ return_result:;
 		LDAP_FREEX( tmpDN, ctx );
 	}
 
-	Debug( LDAP_DEBUG_TRACE, "<= ldap_bv2dn(%s)=%d %s\n", str, rc,
-			ldap_err2string( rc ) );
+	Debug( LDAP_DEBUG_ARGS, "<= ldap_bv2dn(%s)=%d %s\n", str, rc,
+			rc ? ldap_err2string( rc ) : "" );
 	*dn = newDN;
 	
 	return( rc );
@@ -922,7 +911,7 @@ ldap_str2rdn( LDAP_CONST char *str, LDAPRDN *rdn,
 {
 	struct berval	bv;
 
-	assert( str );
+	assert( str != NULL );
 	assert( str[ 0 ] != '\0' );	/* FIXME: is this required? */
 
 	bv.bv_len = strlen( str );
@@ -960,11 +949,11 @@ ldap_bv2rdn_x( struct berval *bv, LDAPRDN *rdn,
 	char		*str;
 	ber_len_t	stoplen;
 	
-	assert( bv );
-	assert( bv->bv_len );
-	assert( bv->bv_val );
+	assert( bv != NULL );
+	assert( bv->bv_len != 0 );
+	assert( bv->bv_val != NULL );
 	assert( rdn || flags & LDAP_DN_SKIP );
-	assert( n );
+	assert( n != NULL );
 
 	str = bv->bv_val;
 	stoplen = bv->bv_len;
@@ -1446,9 +1435,9 @@ str2strval( const char *str, ber_len_t stoplen, struct berval *val, const char *
 	const char 	*p, *end, *startPos, *endPos = NULL;
 	ber_len_t	len, escapes;
 
-	assert( str );
-	assert( val );
-	assert( next );
+	assert( str != NULL );
+	assert( val != NULL );
+	assert( next != NULL );
 
 	*next = NULL;
 	end = str + stoplen;
@@ -1592,9 +1581,9 @@ DCE2strval( const char *str, struct berval *val, const char **next, unsigned fla
 	const char 	*p, *startPos, *endPos = NULL;
 	ber_len_t	len, escapes;
 
-	assert( str );
-	assert( val );
-	assert( next );
+	assert( str != NULL );
+	assert( val != NULL );
+	assert( next != NULL );
 
 	*next = NULL;
 	
@@ -1677,9 +1666,9 @@ IA52strval( const char *str, struct berval *val, const char **next, unsigned fla
 	const char 	*p, *startPos, *endPos = NULL;
 	ber_len_t	len, escapes;
 
-	assert( str );
-	assert( val );
-	assert( next );
+	assert( str != NULL );
+	assert( val != NULL );
+	assert( next != NULL );
 
 	*next = NULL;
 
@@ -1753,9 +1742,9 @@ quotedIA52strval( const char *str, struct berval *val, const char **next, unsign
 	ber_len_t	len;
 	unsigned	escapes = 0;
 
-	assert( str );
-	assert( val );
-	assert( next );
+	assert( str != NULL );
+	assert( val != NULL );
+	assert( next != NULL );
 
 	*next = NULL;
 
@@ -1843,8 +1832,8 @@ hexstr2bin( const char *str, char *c )
 {
 	char	c1, c2;
 
-	assert( str );
-	assert( c );
+	assert( str != NULL );
+	assert( c != NULL );
 
 	c1 = str[ 0 ];
 	c2 = str[ 1 ];
@@ -1885,9 +1874,9 @@ hexstr2binval( const char *str, struct berval *val, const char **next, unsigned
 	ber_len_t	len;
 	ber_len_t	s, d;
 
-	assert( str );
-	assert( val );
-	assert( next );
+	assert( str != NULL );
+	assert( val != NULL );
+	assert( next != NULL );
 
 	*next = NULL;
 
@@ -1987,8 +1976,8 @@ byte2hexpair( const char *val, char *pair )
 {
 	static const char	hexdig[] = "0123456789ABCDEF";
 
-	assert( val );
-	assert( pair );
+	assert( val != NULL );
+	assert( pair != NULL );
 
 	/* 
 	 * we assume the string has enough room for the hex encoding
@@ -2009,8 +1998,8 @@ binval2hexstr( struct berval *val, char *str )
 {
 	ber_len_t	s, d;
 
-	assert( val );
-	assert( str );
+	assert( val != NULL );
+	assert( str != NULL );
 
 	if ( val->bv_len == 0 ) {
 		return( 0 );
@@ -2042,8 +2031,8 @@ strval2strlen( struct berval *val, unsigned flags, ber_len_t *len )
 	int		escaped_ascii_len = LDAP_DN_IS_PRETTY( flags ) ? 2 : 3;
 #endif /* PRETTY_ESCAPE */
 	
-	assert( val );
-	assert( len );
+	assert( val != NULL );
+	assert( len != NULL );
 
 	*len = 0;
 	if ( val->bv_len == 0 ) {
@@ -2120,9 +2109,9 @@ strval2str( struct berval *val, char *str, unsigned flags, ber_len_t *len )
 {
 	ber_len_t	s, d, end;
 
-	assert( val );
-	assert( str );
-	assert( len );
+	assert( val != NULL );
+	assert( str != NULL );
+	assert( len != NULL );
 
 	if ( val->bv_len == 0 ) {
 		*len = 0;
@@ -2219,8 +2208,8 @@ strval2IA5strlen( struct berval *val, unsigned flags, ber_len_t *len )
 	ber_len_t	l;
 	char		*p;
 
-	assert( val );
-	assert( len );
+	assert( val != NULL );
+	assert( len != NULL );
 
 	*len = 0;
 	if ( val->bv_len == 0 ) {
@@ -2261,9 +2250,9 @@ strval2IA5str( struct berval *val, char *str, unsigned flags, ber_len_t *len )
 {
 	ber_len_t	s, d, end;
 
-	assert( val );
-	assert( str );
-	assert( len );
+	assert( val != NULL );
+	assert( str != NULL );
+	assert( len != NULL );
 
 	if ( val->bv_len == 0 ) {
 		*len = 0;
@@ -2309,8 +2298,8 @@ strval2DCEstrlen( struct berval *val, unsigned flags, ber_len_t *len )
 	ber_len_t	l;
 	char		*p;
 
-	assert( val );
-	assert( len );
+	assert( val != NULL );
+	assert( len != NULL );
 
 	*len = 0;
 	if ( val->bv_len == 0 ) {
@@ -2349,9 +2338,9 @@ strval2DCEstr( struct berval *val, char *str, unsigned flags, ber_len_t *len )
 {
 	ber_len_t	s, d;
 
-	assert( val );
-	assert( str );
-	assert( len );
+	assert( val != NULL );
+	assert( str != NULL );
+	assert( len != NULL );
 
 	if ( val->bv_len == 0 ) {
 		*len = 0;
@@ -2395,8 +2384,8 @@ strval2ADstrlen( struct berval *val, unsigned flags, ber_len_t *len )
 	ber_len_t	l;
 	char		*p;
 
-	assert( val );
-	assert( len );
+	assert( val != NULL );
+	assert( len != NULL );
 
 	*len = 0;
 	if ( val->bv_len == 0 ) {
@@ -2435,9 +2424,9 @@ strval2ADstr( struct berval *val, char *str, unsigned flags, ber_len_t *len )
 {
 	ber_len_t	s, d;
 
-	assert( val );
-	assert( str );
-	assert( len );
+	assert( val != NULL );
+	assert( str != NULL );
+	assert( len != NULL );
 
 	if ( val->bv_len == 0 ) {
 		*len = 0;
@@ -2488,9 +2477,9 @@ dn2domain( LDAPDN dn, struct berval *bv, int pos, int *iRDN )
 	/* we are guaranteed there's enough memory in str */
 
 	/* sanity */
-	assert( dn );
-	assert( bv );
-	assert( iRDN );
+	assert( dn != NULL );
+	assert( bv != NULL );
+	assert( iRDN != NULL );
 	assert( *iRDN >= 0 );
 
 	str = bv->bv_val + pos;
@@ -2499,10 +2488,10 @@ dn2domain( LDAPDN dn, struct berval *bv, int pos, int *iRDN )
 		LDAPRDN		rdn;
 		LDAPAVA		*ava;
 
-		assert( dn[ i ] );
+		assert( dn[ i ] != NULL );
 		rdn = dn[ i ];
 
-		assert( rdn[ 0 ] );
+		assert( rdn[ 0 ] != NULL );
 		ava = rdn[ 0 ];
 
 		if ( !LDAP_DN_IS_RDN_DC( rdn ) ) {
@@ -2689,8 +2678,8 @@ rdn2UFNstrlen( LDAPRDN rdn, unsigned flags, ber_len_t *len )
 	int		iAVA;
 	ber_len_t	l = 0;
 
-	assert( rdn );
-	assert( len );
+	assert( rdn != NULL );
+	assert( len != NULL );
 
 	*len = 0;
 
@@ -2768,8 +2757,8 @@ rdn2ADstrlen( LDAPRDN rdn, unsigned flags, ber_len_t *len )
 	int		iAVA;
 	ber_len_t	l = 0;
 
-	assert( rdn );
-	assert( len );
+	assert( rdn != NULL );
+	assert( len != NULL );
 
 	*len = 0;
 
@@ -2849,7 +2838,7 @@ ldap_rdn2str( LDAPRDN rdn, char **str, unsigned flags )
 	struct berval bv;
 	int rc;
 
-	assert( str );
+	assert( str != NULL );
 
 	if((flags & LDAP_DN_FORMAT_MASK) == LDAP_DN_FORMAT_LBER) {
 		return LDAP_PARAM_ERROR;
@@ -2872,7 +2861,7 @@ ldap_rdn2bv_x( LDAPRDN rdn, struct berval *bv, unsigned flags, void *ctx )
 	int		rc, back;
 	ber_len_t	l;
 	
-	assert( bv );
+	assert( bv != NULL );
 
 	bv->bv_len = 0;
 	bv->bv_val = NULL;
@@ -2982,7 +2971,7 @@ int ldap_dn2str( LDAPDN dn, char **str, unsigned flags )
 	struct berval bv;
 	int rc;
 
-	assert( str );
+	assert( str != NULL );
 
 	if((flags & LDAP_DN_FORMAT_MASK) == LDAP_DN_FORMAT_LBER) {
 		return LDAP_PARAM_ERROR;
@@ -3008,11 +2997,11 @@ int ldap_dn2bv_x( LDAPDN dn, struct berval *bv, unsigned flags, void *ctx )
 	int ( *sv2l ) ( struct berval *v, unsigned f, ber_len_t *l );
 	int ( *sv2s ) ( struct berval *v, char *s, unsigned f, ber_len_t *l );
 
-	assert( bv );
+	assert( bv != NULL );
 	bv->bv_len = 0;
 	bv->bv_val = NULL;
 
-	Debug( LDAP_DEBUG_TRACE, "=> ldap_dn2bv(%u)\n", flags, 0, 0 );
+	Debug( LDAP_DEBUG_ARGS, "=> ldap_dn2bv(%u)\n", flags, 0, 0 );
 
 	/* 
 	 * a null dn means an empty dn string 
@@ -3321,8 +3310,8 @@ int ldap_dn2bv_x( LDAPDN dn, struct berval *bv, unsigned flags, void *ctx )
 		return LDAP_PARAM_ERROR;
 	}
 
-	Debug( LDAP_DEBUG_TRACE, "<= ldap_dn2bv(%s)=%d %s\n",
-		bv->bv_val, rc, ldap_err2string( rc ) );
+	Debug( LDAP_DEBUG_ARGS, "<= ldap_dn2bv(%s)=%d %s\n",
+		bv->bv_val, rc, rc ? ldap_err2string( rc ) : "" );
 
 return_results:;
 	return( rc );
@@ -3367,7 +3356,7 @@ ldap_X509dn2bv( void *x509_name, struct berval *bv, LDAPDN_rewrite_func *func,
 
 	struct berval	Val;
 
-	assert( bv );
+	assert( bv != NULL );
 	bv->bv_len = 0;
 	bv->bv_val = NULL;
 
