@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/libraries/liblutil/utils.c,v 1.33 2006/04/01 21:44:42 hallvard Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/liblutil/utils.c,v 1.33.2.1 2006/05/15 17:04:40 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2006 The OpenLDAP Foundation.
@@ -247,14 +247,17 @@ int lutil_parsetime( char *atm, struct lutil_tm *tm )
 		if (tm->tm_sec < 0 || tm->tm_sec > 61) break;
 
 		/* Fractions of seconds */
-		for (i = 0, fracs = 0; isdigit((unsigned char) *ptr); ) {
-			i*=10; i+= *ptr++ - '0';
-			fracs++;
-		}
-		tm->tm_usec = i;
-		if (i) {
-			for (i = fracs; i<6; i++)
-				tm->tm_usec *= 10;
+		if ( *ptr == '.' ) {
+			ptr++;
+			for (i = 0, fracs = 0; isdigit((unsigned char) *ptr); ) {
+				i*=10; i+= *ptr++ - '0';
+				fracs++;
+			}
+			tm->tm_usec = i;
+			if (i) {
+				for (i = fracs; i<6; i++)
+					tm->tm_usec *= 10;
+			}
 		}
 
 		/* Must be UTC */
@@ -362,6 +365,11 @@ lutil_atoux( unsigned *v, const char *s, int x )
 	assert( s != NULL );
 	assert( v != NULL );
 
+	/* strtoul() has an odd interface */
+	if ( s[ 0 ] == '-' ) {
+		return -1;
+	}
+
 	u = strtoul( s, &next, x );
 	if ( next == s || next[ 0 ] != '\0' ) {
 		return -1;
@@ -404,6 +412,11 @@ lutil_atoulx( unsigned long *v, const char *s, int x )
 	assert( s != NULL );
 	assert( v != NULL );
 
+	/* strtoul() has an odd interface */
+	if ( s[ 0 ] == '-' ) {
+		return -1;
+	}
+
 	ul = strtoul( s, &next, x );
 	if ( next == s || next[ 0 ] != '\0' ) {
 		return -1;
@@ -433,6 +446,11 @@ lutil_parse_time(
 		unsigned long	u;
 		char		*what;
 
+		/* strtoul() has an odd interface */
+		if ( s[ 0 ] == '-' ) {
+			return -1;
+		}
+
 		u = strtoul( s, &next, 10 );
 		if ( next == s ) {
 			return -1;
