@@ -1,8 +1,8 @@
 /* abandon.c - decode and handle an ldap abandon operation */
-/* $OpenLDAP: pkg/ldap/servers/slapd/abandon.c,v 1.45.2.2 2005/01/20 18:03:53 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/abandon.c,v 1.52 2006/01/03 22:12:12 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2005 The OpenLDAP Foundation.
+ * Copyright 1998-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,7 +36,6 @@ do_abandon( Operation *op, SlapReply *rs )
 {
 	ber_int_t	id;
 	Operation	*o;
-	int		i;
 
 	Debug( LDAP_DEBUG_TRACE, "do_abandon\n", 0, 0, 0 );
 
@@ -52,6 +51,9 @@ do_abandon( Operation *op, SlapReply *rs )
 		return SLAPD_DISCONNECT;
 	}
 
+	Statslog( LDAP_DEBUG_STATS, "%s ABANDON msg=%ld\n",
+		op->o_log_prefix, (long) id, 0, 0, 0 );
+
 	if( get_ctrls( op, rs, 0 ) != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_ANY, "do_abandon: get_ctrls failed\n", 0, 0 ,0 );
 		return rs->sr_err;
@@ -75,27 +77,29 @@ do_abandon( Operation *op, SlapReply *rs )
 	LDAP_STAILQ_FOREACH( o, &op->o_conn->c_ops, o_next ) {
 		if ( o->o_msgid == id ) {
 			o->o_abandon = 1;
-			goto done;
+			break;
 		}
 	}
 
-	LDAP_STAILQ_FOREACH( o, &op->o_conn->c_pending_ops, o_next ) {
-		if ( o->o_msgid == id ) {
-			LDAP_STAILQ_REMOVE( &op->o_conn->c_pending_ops,
-				o, slap_op, o_next );
-			LDAP_STAILQ_NEXT(o, o_next) = NULL;
-			op->o_conn->c_n_ops_pending--;
-			slap_op_free( o );
-			goto done;
+	if ( o ) {
+		op->orn_msgid = id;
+
+		op->o_bd = frontendDB;
+		rs->sr_err = frontendDB->be_abandon( op, rs );
+
+	} else {
+		LDAP_STAILQ_FOREACH( o, &op->o_conn->c_pending_ops, o_next ) {
+			if ( o->o_msgid == id ) {
+				LDAP_STAILQ_REMOVE( &op->o_conn->c_pending_ops,
+					o, slap_op, o_next );
+				LDAP_STAILQ_NEXT(o, o_next) = NULL;
+				op->o_conn->c_n_ops_pending--;
+				slap_op_free( o );
+				break;
+			}
 		}
 	}
 
-done:
-	op->orn_msgid = id;
-
-	op->o_bd = frontendDB;
-	rs->sr_err = frontendDB->be_abandon( op, rs );
-
 	ldap_pvt_thread_mutex_unlock( &op->o_conn->c_mutex );
 
 	Debug( LDAP_DEBUG_TRACE, "do_abandon: op=%ld %sfound\n",
@@ -106,10 +110,7 @@ done:
 int
 fe_op_abandon( Operation *op, SlapReply *rs )
 {
-	int i;
-
-	for ( i = 0; i < nbackends; i++ ) {
-		op->o_bd = &backends[i];
+	LDAP_STAILQ_FOREACH( op->o_bd, &backendDB, be_next ) {
 		if ( op->o_bd->be_abandon ) {
 			(void)op->o_bd->be_abandon( op, rs );
 		}
