@@ -1,8 +1,8 @@
 /* at.c - routines for dealing with attribute types */
-/* $OpenLDAP: pkg/ldap/servers/slapd/at.c,v 1.62.2.2 2005/01/24 21:28:51 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/at.c,v 1.84 2006/03/31 20:05:01 hyc Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2005 The OpenLDAP Foundation.
+ * Copyright 1998-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -58,8 +58,8 @@ struct aindexrec {
 
 static Avlnode	*attr_index = NULL;
 static Avlnode	*attr_cache = NULL;
-static LDAP_SLIST_HEAD(ATList, slap_attribute_type) attr_list
-	= LDAP_SLIST_HEAD_INITIALIZER(&attr_list);
+static LDAP_STAILQ_HEAD(ATList, slap_attribute_type) attr_list
+	= LDAP_STAILQ_HEAD_INITIALIZER(attr_list);
 
 int at_oc_cache;
 
@@ -210,11 +210,35 @@ at_destroy( void )
 	AttributeType *a;
 	avl_free(attr_index, ldap_memfree);
 
-	while( !LDAP_SLIST_EMPTY(&attr_list) ) {
-		a = LDAP_SLIST_FIRST(&attr_list);
-		LDAP_SLIST_REMOVE_HEAD(&attr_list, sat_next);
+	while( !LDAP_STAILQ_EMPTY(&attr_list) ) {
+		a = LDAP_STAILQ_FIRST(&attr_list);
+		LDAP_STAILQ_REMOVE_HEAD(&attr_list, sat_next);
 
-		if (a->sat_subtypes) ldap_memfree(a->sat_subtypes);
+		if ( a->sat_equality ) {
+			MatchingRule	*mr;
+
+			mr = mr_find( a->sat_equality->smr_oid );
+			assert( mr != NULL );
+			if ( mr != a->sat_equality ) {
+				ch_free( a->sat_equality );
+				a->sat_equality = NULL;
+			}
+		}
+
+		assert( a->sat_syntax != NULL );
+		if ( a->sat_syntax != NULL ) {
+			Syntax		*syn;
+
+			syn = syn_find( a->sat_syntax->ssyn_oid );
+			assert( syn != NULL );
+			if ( syn != a->sat_syntax ) {
+				ch_free( a->sat_syntax );
+				a->sat_syntax = NULL;
+			}
+		}
+
+		if ( a->sat_oidmacro ) ldap_memfree( a->sat_oidmacro );
+		if ( a->sat_subtypes ) ldap_memfree( a->sat_subtypes );
 		ad_destroy(a->sat_ad);
 		ldap_pvt_thread_mutex_destroy(&a->sat_ad_mutex);
 		ldap_attributetype_free((LDAPAttributeType *)a);
@@ -223,14 +247,18 @@ at_destroy( void )
 	if ( slap_schema.si_at_undefined ) {
 		ad_destroy(slap_schema.si_at_undefined->sat_ad);
 	}
+
+	if ( slap_schema.si_at_proxied ) {
+		ad_destroy(slap_schema.si_at_proxied->sat_ad);
+	}
 }
 
 int
 at_start( AttributeType **at )
 {
-	assert( at );
+	assert( at != NULL );
 
-	*at = LDAP_SLIST_FIRST(&attr_list);
+	*at = LDAP_STAILQ_FIRST(&attr_list);
 
 	return (*at != NULL);
 }
@@ -238,27 +266,77 @@ at_start( AttributeType **at )
 int
 at_next( AttributeType **at )
 {
-	assert( at );
+	assert( at != NULL );
 
 #if 1	/* pedantic check */
 	{
 		AttributeType *tmp = NULL;
 
-		LDAP_SLIST_FOREACH(tmp,&attr_list,sat_next) {
+		LDAP_STAILQ_FOREACH(tmp,&attr_list,sat_next) {
 			if ( tmp == *at ) {
 				break;
 			}
 		}
 
-		assert( tmp );
+		assert( tmp != NULL );
 	}
 #endif
 
-	*at = LDAP_SLIST_NEXT(*at,sat_next);
+	*at = LDAP_STAILQ_NEXT(*at,sat_next);
 
 	return (*at != NULL);
 }
-	
+
+/*
+ * check whether the two attributeTypes actually __are__ identical,
+ * or rather inconsistent
+ */
+static int
+at_check_dup(
+	AttributeType		*sat,
+	AttributeType		*new_sat )
+{
+	if ( new_sat->sat_oid != NULL ) {
+		if ( sat->sat_oid == NULL ) {
+			return SLAP_SCHERR_ATTR_INCONSISTENT;
+		}
+
+		if ( strcmp( sat->sat_oid, new_sat->sat_oid ) != 0 ) {
+			return SLAP_SCHERR_ATTR_INCONSISTENT;
+		}
+
+	} else {
+		if ( sat->sat_oid != NULL ) {
+			return SLAP_SCHERR_ATTR_INCONSISTENT;
+		}
+	}
+
+	if ( new_sat->sat_names ) {
+		int	i;
+
+		if ( sat->sat_names == NULL ) {
+			return SLAP_SCHERR_ATTR_INCONSISTENT;
+		}
+
+		for ( i = 0; new_sat->sat_names[ i ]; i++ ) {
+			if ( sat->sat_names[ i ] == NULL ) {
+				return SLAP_SCHERR_ATTR_INCONSISTENT;
+			}
+			
+			if ( strcasecmp( sat->sat_names[ i ],
+					new_sat->sat_names[ i ] ) != 0 )
+			{
+				return SLAP_SCHERR_ATTR_INCONSISTENT;
+			}
+		}
+	} else {
+		if ( sat->sat_names != NULL ) {
+			return SLAP_SCHERR_ATTR_INCONSISTENT;
+		}
+	}
+
+	return SLAP_SCHERR_ATTR_DUP;
+}
 
 
 static int
@@ -267,39 +345,79 @@ at_insert(
     const char		**err )
 {
 	struct aindexrec	*air;
-	char			**names;
+	char			**names = NULL;
 
-	LDAP_SLIST_NEXT( sat, sat_next ) = NULL;
-	LDAP_SLIST_INSERT_HEAD( &attr_list, sat, sat_next );
 
 	if ( sat->sat_oid ) {
 		air = (struct aindexrec *)
 			ch_calloc( 1, sizeof(struct aindexrec) );
-		air->air_name.bv_val = sat->sat_oid;
-		air->air_name.bv_len = strlen(sat->sat_oid);
+		ber_str2bv( sat->sat_oid, 0, 0, &air->air_name );
 		air->air_at = sat;
 		if ( avl_insert( &attr_index, (caddr_t) air,
-		                 attr_index_cmp, avl_dup_error ) ) {
+		                 attr_index_cmp, avl_dup_error ) )
+		{
+			AttributeType	*old_sat;
+			int		rc;
+
 			*err = sat->sat_oid;
-			ldap_memfree(air);
-			return SLAP_SCHERR_ATTR_DUP;
+
+			old_sat = at_bvfind( &air->air_name );
+			assert( old_sat != NULL );
+			rc = at_check_dup( old_sat, sat );
+
+			ldap_memfree( air );
+
+			return rc;
 		}
 		/* FIX: temporal consistency check */
-		at_bvfind(&air->air_name);
+		at_bvfind( &air->air_name );
 	}
 
-	if ( (names = sat->sat_names) ) {
+	names = sat->sat_names;
+	if ( names ) {
 		while ( *names ) {
 			air = (struct aindexrec *)
 				ch_calloc( 1, sizeof(struct aindexrec) );
-			air->air_name.bv_val = *names;
-			air->air_name.bv_len = strlen(*names);
+			ber_str2bv( *names, 0, 0, &air->air_name );
 			air->air_at = sat;
 			if ( avl_insert( &attr_index, (caddr_t) air,
-			                 attr_index_cmp, avl_dup_error ) ) {
+			                 attr_index_cmp, avl_dup_error ) )
+			{
+				AttributeType	*old_sat;
+				int		rc;
+
 				*err = *names;
+
+				old_sat = at_bvfind( &air->air_name );
+				assert( old_sat != NULL );
+				rc = at_check_dup( old_sat, sat );
+
 				ldap_memfree(air);
-				return SLAP_SCHERR_ATTR_DUP;
+
+				while ( names > sat->sat_names ) {
+					struct aindexrec	tmpair;
+
+					names--;
+					ber_str2bv( *names, 0, 0, &tmpair.air_name );
+					tmpair.air_at = sat;
+					air = (struct aindexrec *)avl_delete( &attr_index,
+						(caddr_t)&tmpair, attr_index_cmp );
+					assert( air != NULL );
+					ldap_memfree( air );
+				}
+
+				if ( sat->sat_oid ) {
+					struct aindexrec	tmpair;
+
+					ber_str2bv( sat->sat_oid, 0, 0, &tmpair.air_name );
+					tmpair.air_at = sat;
+					air = (struct aindexrec *)avl_delete( &attr_index,
+						(caddr_t)&tmpair, attr_index_cmp );
+					assert( air != NULL );
+					ldap_memfree( air );
+				}
+
+				return rc;
 			}
 			/* FIX: temporal consistency check */
 			at_bvfind(&air->air_name);
@@ -307,23 +425,46 @@ at_insert(
 		}
 	}
 
+	if ( sat->sat_oid ) {
+		slap_ad_undef_promote( sat->sat_oid, sat );
+	}
+
+	names = sat->sat_names;
+	if ( names ) {
+		while ( *names ) {
+			slap_ad_undef_promote( *names, sat );
+			names++;
+		}
+	}
+
+	LDAP_STAILQ_INSERT_TAIL( &attr_list, sat, sat_next );
+
 	return 0;
 }
 
 int
 at_add(
-    LDAPAttributeType	*at,
-    const char		**err )
+	LDAPAttributeType	*at,
+	int			user,
+	AttributeType		**rsat,
+	const char		**err )
 {
-	AttributeType	*sat;
-	MatchingRule	*mr;
-	Syntax		*syn;
+	AttributeType	*sat = NULL;
+	MatchingRule	*mr = NULL;
+	Syntax		*syn = NULL;
 	int		i;
-	int		code;
-	char	*cname;
-	char	*oid;
+	int		code = LDAP_SUCCESS;
+	char		*cname = NULL;
+	char		*oidm = NULL;
+
+	if ( !at->at_oid ) {
+		*err = "";
+		return SLAP_SCHERR_ATTR_INCOMPLETE;
+	}
 
 	if ( !OID_LEADCHAR( at->at_oid[0] )) {
+		char	*oid;
+
 		/* Expand OID macros */
 		oid = oidm_find( at->at_oid );
 		if ( !oid ) {
@@ -331,17 +472,20 @@ at_add(
 			return SLAP_SCHERR_OIDM;
 		}
 		if ( oid != at->at_oid ) {
-			ldap_memfree( at->at_oid );
+			oidm = at->at_oid;
 			at->at_oid = oid;
 		}
 	}
 
 	if ( at->at_syntax_oid && !OID_LEADCHAR( at->at_syntax_oid[0] )) {
+		char	*oid;
+
 		/* Expand OID macros */
 		oid = oidm_find( at->at_syntax_oid );
 		if ( !oid ) {
 			*err = at->at_syntax_oid;
-			return SLAP_SCHERR_OIDM;
+			code = SLAP_SCHERR_OIDM;
+			goto error_return;
 		}
 		if ( oid != at->at_syntax_oid ) {
 			ldap_memfree( at->at_syntax_oid );
@@ -355,36 +499,37 @@ at_add(
 		for( i=0; at->at_names[i]; i++ ) {
 			if( !slap_valid_descr( at->at_names[i] ) ) {
 				*err = at->at_names[i];
-				return SLAP_SCHERR_BAD_DESCR;
+				code = SLAP_SCHERR_BAD_DESCR;
+				goto error_return;
 			}
 		}
 
 		cname = at->at_names[0];
 
-	} else if ( at->at_oid ) {
+	} else {
 		cname = at->at_oid;
 
-	} else {
-		*err = "";
-		return SLAP_SCHERR_ATTR_INCOMPLETE;
 	}
 
 	*err = cname;
 
 	if ( !at->at_usage && at->at_no_user_mod ) {
 		/* user attribute must be modifable */
-		return SLAP_SCHERR_ATTR_BAD_USAGE;
+		code = SLAP_SCHERR_ATTR_BAD_USAGE;
+		goto error_return;
 	}
 
 	if ( at->at_collective ) {
 		if( at->at_usage ) {
 			/* collective attributes cannot be operational */
-			return SLAP_SCHERR_ATTR_BAD_USAGE;
+			code = SLAP_SCHERR_ATTR_BAD_USAGE;
+			goto error_return;
 		}
 
 		if( at->at_single_value ) {
 			/* collective attributes cannot be single-valued */
-			return SLAP_SCHERR_ATTR_BAD_USAGE;
+			code = SLAP_SCHERR_ATTR_BAD_USAGE;
+			goto error_return;
 		}
 	}
 
@@ -393,6 +538,7 @@ at_add(
 
 	sat->sat_cname.bv_val = cname;
 	sat->sat_cname.bv_len = strlen( cname );
+	sat->sat_oidmacro = oidm;
 	ldap_pvt_thread_mutex_init(&sat->sat_ad_mutex);
 
 	if ( at->at_sup_oid ) {
@@ -400,28 +546,33 @@ at_add(
 
 		if ( supsat == NULL ) {
 			*err = at->at_sup_oid;
-			return SLAP_SCHERR_ATTR_NOT_FOUND;
+			code = SLAP_SCHERR_ATTR_NOT_FOUND;
+			goto error_return;
 		}
 
 		sat->sat_sup = supsat;
 
 		if ( at_append_to_list(sat, &supsat->sat_subtypes) ) {
-			return SLAP_SCHERR_OUTOFMEM;
+			code = SLAP_SCHERR_OUTOFMEM;
+			goto error_return;
 		}
 
 		if ( sat->sat_usage != supsat->sat_usage ) {
 			/* subtypes must have same usage as their SUP */
-			return SLAP_SCHERR_ATTR_BAD_USAGE;
+			code = SLAP_SCHERR_ATTR_BAD_USAGE;
+			goto error_return;
 		}
 
 		if ( supsat->sat_obsolete && !sat->sat_obsolete ) {
 			/* subtypes must be obsolete if super is */
-			return SLAP_SCHERR_ATTR_BAD_SUP;
+			code = SLAP_SCHERR_ATTR_BAD_SUP;
+			goto error_return;
 		}
 
 		if ( sat->sat_flags & SLAP_AT_FINAL ) {
 			/* cannot subtype a "final" attribute type */
-			return SLAP_SCHERR_ATTR_BAD_SUP;
+			code = SLAP_SCHERR_ATTR_BAD_SUP;
+			goto error_return;
 		}
 	}
 
@@ -438,21 +589,47 @@ at_add(
 		sat->sat_substr = sat->sat_sup->sat_substr;
 	}
 
+	/*
+	 * check for X-ORDERED attributes
+	 */
+	if ( sat->sat_extensions ) {
+		for (i=0; sat->sat_extensions[i]; i++) {
+			if (!strcasecmp( sat->sat_extensions[i]->lsei_name,
+				"X-ORDERED" ) && sat->sat_extensions[i]->lsei_values ) {
+				if ( !strcasecmp( sat->sat_extensions[i]->lsei_values[0],
+					"VALUES" )) {
+					sat->sat_flags |= SLAP_AT_ORDERED_VAL;
+					break;
+				} else if ( !strcasecmp( sat->sat_extensions[i]->lsei_values[0],
+					"SIBLINGS" )) {
+					sat->sat_flags |= SLAP_AT_ORDERED_SIB;
+					break;
+				}
+			}
+		}
+	}
+
+	if ( !user )
+		sat->sat_flags |= SLAP_AT_HARDCODE;
+
 	if ( at->at_syntax_oid ) {
 		syn = syn_find(sat->sat_syntax_oid);
 		if ( syn == NULL ) {
 			*err = sat->sat_syntax_oid;
-			return SLAP_SCHERR_SYN_NOT_FOUND;
+			code = SLAP_SCHERR_SYN_NOT_FOUND;
+			goto error_return;
 		}
 
 		if( sat->sat_syntax != NULL && sat->sat_syntax != syn ) {
-			return SLAP_SCHERR_ATTR_BAD_SUP;
+			code = SLAP_SCHERR_ATTR_BAD_SUP;
+			goto error_return;
 		}
 
 		sat->sat_syntax = syn;
 
 	} else if ( sat->sat_syntax == NULL ) {
-		return SLAP_SCHERR_ATTR_INCOMPLETE;
+		code = SLAP_SCHERR_ATTR_INCOMPLETE;
+		goto error_return;
 	}
 
 	if ( sat->sat_equality_oid ) {
@@ -460,18 +637,21 @@ at_add(
 
 		if( mr == NULL ) {
 			*err = sat->sat_equality_oid;
-			return SLAP_SCHERR_MR_NOT_FOUND;
+			code = SLAP_SCHERR_MR_NOT_FOUND;
+			goto error_return;
 		}
 
 		if(( mr->smr_usage & SLAP_MR_EQUALITY ) != SLAP_MR_EQUALITY ) {
 			*err = sat->sat_equality_oid;
-			return SLAP_SCHERR_ATTR_BAD_MR;
+			code = SLAP_SCHERR_ATTR_BAD_MR;
+			goto error_return;
 		}
 
 		if( sat->sat_syntax != mr->smr_syntax ) {
 			if( mr->smr_compat_syntaxes == NULL ) {
 				*err = sat->sat_equality_oid;
-				return SLAP_SCHERR_ATTR_BAD_MR;
+				code = SLAP_SCHERR_ATTR_BAD_MR;
+				goto error_return;
 			}
 
 			for(i=0; mr->smr_compat_syntaxes[i]; i++) {
@@ -483,7 +663,8 @@ at_add(
 
 			if( i >= 0 ) {
 				*err = sat->sat_equality_oid;
-				return SLAP_SCHERR_ATTR_BAD_MR;
+				code = SLAP_SCHERR_ATTR_BAD_MR;
+				goto error_return;
 			}
 		}
 
@@ -494,25 +675,29 @@ at_add(
 	if ( sat->sat_ordering_oid ) {
 		if( !sat->sat_equality ) {
 			*err = sat->sat_ordering_oid;
-			return SLAP_SCHERR_ATTR_BAD_MR;
+			code = SLAP_SCHERR_ATTR_BAD_MR;
+			goto error_return;
 		}
 
 		mr = mr_find(sat->sat_ordering_oid);
 
 		if( mr == NULL ) {
 			*err = sat->sat_ordering_oid;
-			return SLAP_SCHERR_MR_NOT_FOUND;
+			code = SLAP_SCHERR_MR_NOT_FOUND;
+			goto error_return;
 		}
 
 		if(( mr->smr_usage & SLAP_MR_ORDERING ) != SLAP_MR_ORDERING ) {
 			*err = sat->sat_ordering_oid;
-			return SLAP_SCHERR_ATTR_BAD_MR;
+			code = SLAP_SCHERR_ATTR_BAD_MR;
+			goto error_return;
 		}
 
 		if( sat->sat_syntax != mr->smr_syntax ) {
 			if( mr->smr_compat_syntaxes == NULL ) {
 				*err = sat->sat_ordering_oid;
-				return SLAP_SCHERR_ATTR_BAD_MR;
+				code = SLAP_SCHERR_ATTR_BAD_MR;
+				goto error_return;
 			}
 
 			for(i=0; mr->smr_compat_syntaxes[i]; i++) {
@@ -524,7 +709,8 @@ at_add(
 
 			if( i >= 0 ) {
 				*err = sat->sat_ordering_oid;
-				return SLAP_SCHERR_ATTR_BAD_MR;
+				code = SLAP_SCHERR_ATTR_BAD_MR;
+				goto error_return;
 			}
 		}
 
@@ -534,19 +720,22 @@ at_add(
 	if ( sat->sat_substr_oid ) {
 		if( !sat->sat_equality ) {
 			*err = sat->sat_substr_oid;
-			return SLAP_SCHERR_ATTR_BAD_MR;
+			code = SLAP_SCHERR_ATTR_BAD_MR;
+			goto error_return;
 		}
 
 		mr = mr_find(sat->sat_substr_oid);
 
 		if( mr == NULL ) {
 			*err = sat->sat_substr_oid;
-			return SLAP_SCHERR_MR_NOT_FOUND;
+			code = SLAP_SCHERR_MR_NOT_FOUND;
+			goto error_return;
 		}
 
 		if(( mr->smr_usage & SLAP_MR_SUBSTR ) != SLAP_MR_SUBSTR ) {
 			*err = sat->sat_substr_oid;
-			return SLAP_SCHERR_ATTR_BAD_MR;
+			code = SLAP_SCHERR_ATTR_BAD_MR;
+			goto error_return;
 		}
 
 		/* due to funky LDAP builtin substring rules,
@@ -557,7 +746,8 @@ at_add(
 		if( sat->sat_syntax != sat->sat_equality->smr_syntax ) {
 			if( sat->sat_equality->smr_compat_syntaxes == NULL ) {
 				*err = sat->sat_substr_oid;
-				return SLAP_SCHERR_ATTR_BAD_MR;
+				code = SLAP_SCHERR_ATTR_BAD_MR;
+				goto error_return;
 			}
 
 			for(i=0; sat->sat_equality->smr_compat_syntaxes[i]; i++) {
@@ -571,18 +761,36 @@ at_add(
 
 			if( i >= 0 ) {
 				*err = sat->sat_substr_oid;
-				return SLAP_SCHERR_ATTR_BAD_MR;
+				code = SLAP_SCHERR_ATTR_BAD_MR;
+				goto error_return;
 			}
 		}
 
 		sat->sat_substr = mr;
 	}
 
-	code = at_insert(sat,err);
+	code = at_insert( sat, err );
+	if ( code != 0 ) {
+error_return:;
+		if ( sat ) {
+			ldap_pvt_thread_mutex_destroy( &sat->sat_ad_mutex );
+			ch_free( sat );
+		}
+
+		if ( oidm ) {
+			SLAP_FREE( at->at_oid );
+			at->at_oid = oidm;
+		}
+
+	} else if ( rsat ) {
+		*rsat = sat;
+	}
+
 	return code;
 }
 
 #ifdef LDAP_DEBUG
+#ifdef SLAPD_UNUSED
 static int
 at_index_printnode( void *v_air, void *ignore )
 {
@@ -600,6 +808,64 @@ at_index_print( void )
 	(void) avl_apply( attr_index, at_index_printnode, 0, -1, AVL_INORDER );
 }
 #endif
+#endif
+
+void
+at_unparse( BerVarray *res, AttributeType *start, AttributeType *end, int sys )
+{
+	AttributeType *at;
+	int i, num;
+	struct berval bv, *bva = NULL, idx;
+	char ibuf[32];
+
+	if ( !start )
+		start = LDAP_STAILQ_FIRST( &attr_list );
+
+	/* count the result size */
+	i = 0;
+	for ( at=start; at; at=LDAP_STAILQ_NEXT(at, sat_next)) {
+		if ( sys && !(at->sat_flags & SLAP_AT_HARDCODE)) continue;
+		i++;
+		if ( at == end ) break;
+	}
+	if (!i) return;
+
+	num = i;
+	bva = ch_malloc( (num+1) * sizeof(struct berval) );
+	BER_BVZERO( bva );
+	idx.bv_val = ibuf;
+	if ( sys ) {
+		idx.bv_len = 0;
+		ibuf[0] = '\0';
+	}
+	i = 0;
+	for ( at=start; at; at=LDAP_STAILQ_NEXT(at, sat_next)) {
+		LDAPAttributeType lat, *latp;
+		if ( sys && !(at->sat_flags & SLAP_AT_HARDCODE)) continue;
+		if ( at->sat_oidmacro ) {
+			lat = at->sat_atype;
+			lat.at_oid = at->sat_oidmacro;
+			latp = &lat;
+		} else {
+			latp = &at->sat_atype;
+		}
+		if ( ldap_attributetype2bv( latp, &bv ) == NULL ) {
+			ber_bvarray_free( bva );
+		}
+		if ( !sys ) {
+			idx.bv_len = sprintf(idx.bv_val, "{%d}", i);
+		}
+		bva[i].bv_len = idx.bv_len + bv.bv_len;
+		bva[i].bv_val = ch_malloc( bva[i].bv_len + 1 );
+		strcpy( bva[i].bv_val, ibuf );
+		strcpy( bva[i].bv_val + idx.bv_len, bv.bv_val );
+		i++;
+		bva[i].bv_val = NULL;
+		ldap_memfree( bv.bv_val );
+		if ( at == end ) break;
+	}
+	*res = bva;
+}
 
 int
 at_schema_info( Entry *e )
@@ -609,15 +875,14 @@ at_schema_info( Entry *e )
 	struct berval	val;
 	struct berval	nval;
 
-	LDAP_SLIST_FOREACH(at,&attr_list,sat_next) {
+	LDAP_STAILQ_FOREACH(at,&attr_list,sat_next) {
 		if( at->sat_flags & SLAP_AT_HIDE ) continue;
 
 		if ( ldap_attributetype2bv( &at->sat_atype, &val ) == NULL ) {
 			return -1;
 		}
 
-		nval.bv_val = at->sat_oid;
-		nval.bv_len = strlen(at->sat_oid);
+		ber_str2bv( at->sat_oid, 0, 0, &nval );
 
 		if( attr_merge_one( e, ad_attributeTypes, &val, &nval ) )
 		{
