@@ -1,8 +1,8 @@
 /* dn2id.c - routines to deal with the dn2id index */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/dn2id.c,v 1.106.2.2 2005/01/24 21:28:52 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/dn2id.c,v 1.137 2006/01/03 22:12:17 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2005 The OpenLDAP Foundation.
+ * Copyright 2000-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -178,7 +178,7 @@ bdb_dn2id_delete(
 #endif
 	{
 		buf[0] = DN_SUBTREE_PREFIX;
-		rc = db->del( db, txn, &key, 0 );
+		rc = bdb_idl_delete_key( op->o_bd, db, txn, &key, e->e_id );
 		if( rc != 0 ) {
 			Debug( LDAP_DEBUG_ANY,
 			"=> bdb_dn2id_delete: subtree (%s) delete failed: %d\n",
@@ -361,6 +361,7 @@ bdb_dn2idl(
 	((char *)key.data)[0] = prefix;
 	AC_MEMCPY( &((char *)key.data)[1], e->e_nname.bv_val, key.size - 1 );
 
+	BDB_IDL_ZERO( ids );
 	rc = bdb_idl_fetch_key( op->o_bd, db, NULL, &key, ids, NULL, 0 );
 
 	if( rc != 0 ) {
@@ -380,10 +381,7 @@ bdb_dn2idl(
 }
 
 #else	/* BDB_HIER */
-/* Experimental management routines for a hierarchically structured database.
- *
- * Unsupported! Use at your own risk!
- * -- Howard Chu, Symas Corp. 2003.
+/* Management routines for a hierarchically structured database.
  *
  * Instead of a ldbm-style dn2id database, we use a hierarchical one. Each
  * entry in this database is a struct diskNode, keyed by entryID and with
@@ -401,9 +399,9 @@ bdb_dn2idl(
  */
 typedef struct diskNode {
 	unsigned char nrdnlen[2];
-	unsigned char nrdn[1];
-	unsigned char rdn[1];
-	unsigned char entryID[sizeof(ID)];
+	char nrdn[1];
+	char rdn[1];                        /* variable placement */
+	unsigned char entryID[sizeof(ID)];  /* variable placement */
 } diskNode;
 
 /* This function constructs a full DN for a given entry.
@@ -417,6 +415,9 @@ int hdb_fix_dn(
 	char *ptr, *nptr;
 	int max = 0;
 
+	if ( !e->e_id )
+		return 0;
+
 	/* count length of all DN components */
 	for ( ei = BEI(e); ei && ei->bei_id; ei=ei->bei_parent ) {
 		rlen += ei->bei_rdn.bv_len + 1;
@@ -497,16 +498,17 @@ hdb_dn2id_add(
 
 	DBTzero(&key);
 	DBTzero(&data);
-	key.data = &nid;
 	key.size = sizeof(ID);
 	key.flags = DB_DBT_USERMEM;
 	BDB_ID2DISK( eip->bei_id, &nid );
 
+	key.data = &nid;
+
 	/* Need to make dummy root node once. Subsequent attempts
 	 * will fail harmlessly.
 	 */
 	if ( eip->bei_id == 0 ) {
-		diskNode dummy = {0};
+		diskNode dummy = {{0, 0}, "", "", ""};
 		data.data = &dummy;
 		data.size = sizeof(diskNode);
 		data.flags = DB_DBT_USERMEM;
@@ -514,9 +516,6 @@ hdb_dn2id_add(
 		db->put( db, txn, &key, &data, DB_NODUPDATA );
 	}
 
-	if ( bdb->bi_idl_cache_size ) {
-		bdb_idl_cache_del( bdb, db, &key );
-	}
 	data.data = d;
 	data.size = sizeof(diskNode) + rlen + nrlen;
 	data.flags = DB_DBT_USERMEM;
@@ -531,6 +530,21 @@ hdb_dn2id_add(
 		rc = db->put( db, txn, &key, &data, DB_NODUPDATA );
 	}
 
+	/* Update all parents' IDL cache entries */
+	if ( rc == 0 && bdb->bi_idl_cache_size ) {
+		ID tmp[2];
+		char *ptr = ((char *)&tmp[1])-1;
+		key.data = ptr;
+		key.size = sizeof(ID)+1;
+		tmp[1] = eip->bei_id;
+		*ptr = DN_ONE_PREFIX;
+		bdb_idl_cache_add_id( bdb, db, &key, e->e_id );
+		*ptr = DN_SUBTREE_PREFIX;
+		for (; eip && eip->bei_parent->bei_id; eip = eip->bei_parent) {
+			tmp[1] = eip->bei_id;
+			bdb_idl_cache_add_id( bdb, db, &key, e->e_id );
+		}
+	}
 	op->o_tmpfree( d, op->o_tmpmemctx );
 
 	return rc;
@@ -548,13 +562,13 @@ hdb_dn2id_delete(
 	DBT		key, data;
 	DBC	*cursor;
 	diskNode *d;
-	int rc, nrlen;
+	int rc;
 	ID	nid;
+	unsigned char dlen[2];
 
 	DBTzero(&key);
 	key.size = sizeof(ID);
 	key.ulen = key.size;
-	key.data = &nid;
 	key.flags = DB_DBT_USERMEM;
 	BDB_ID2DISK( eip->bei_id, &nid );
 
@@ -564,22 +578,23 @@ hdb_dn2id_delete(
 	data.dlen = data.size;
 	data.flags = DB_DBT_USERMEM | DB_DBT_PARTIAL;
 
-	if ( bdb->bi_idl_cache_size ) {
-		bdb_idl_cache_del( bdb, db, &key );
-	}
+	key.data = &nid;
 	rc = db->cursor( db, txn, &cursor, bdb->bi_db_opflags );
 	if ( rc ) return rc;
 
 	d = op->o_tmpalloc( data.size, op->o_tmpmemctx );
 	d->nrdnlen[1] = BEI(e)->bei_nrdn.bv_len & 0xff;
 	d->nrdnlen[0] = (BEI(e)->bei_nrdn.bv_len >> 8) | 0x80;
+	dlen[0] = d->nrdnlen[0];
+	dlen[1] = d->nrdnlen[1];
 	strcpy( d->nrdn, BEI(e)->bei_nrdn.bv_val );
 	data.data = d;
 
 	/* Delete our ID from the parent's list */
 	rc = cursor->c_get( cursor, &key, &data, DB_GET_BOTH_RANGE );
 	if ( rc == 0 ) {
-		if ( !strcmp( d->nrdn, BEI(e)->bei_nrdn.bv_val ))
+		if ( dlen[1] == d->nrdnlen[1] && dlen[0] == d->nrdnlen[0] &&
+			!strcmp( d->nrdn, BEI(e)->bei_nrdn.bv_val ))
 			rc = cursor->c_del( cursor, 0 );
 		else
 			rc = DB_NOTFOUND;
@@ -598,6 +613,21 @@ hdb_dn2id_delete(
 	cursor->c_close( cursor );
 	op->o_tmpfree( d, op->o_tmpmemctx );
 
+	/* Delete IDL cache entries */
+	if ( rc == 0 && bdb->bi_idl_cache_size ) {
+		ID tmp[2];
+		char *ptr = ((char *)&tmp[1])-1;
+		key.data = ptr;
+		key.size = sizeof(ID)+1;
+		tmp[1] = eip->bei_id;
+		*ptr = DN_ONE_PREFIX;
+		bdb_idl_cache_del_id( bdb, db, &key, e->e_id );
+		*ptr = DN_SUBTREE_PREFIX;
+		for (; eip && eip->bei_parent->bei_id; eip = eip->bei_parent) {
+			tmp[1] = eip->bei_id;
+			bdb_idl_cache_del_id( bdb, db, &key, e->e_id );
+		}
+	}
 	return rc;
 }
 
@@ -616,7 +646,8 @@ hdb_dn2id(
 	int		rc = 0, nrlen;
 	diskNode *d;
 	char	*ptr;
-	ID idp;
+	unsigned char dlen[2];
+	ID idp, parentID;
 
 	nrlen = dn_rdnlen( op->o_bd, in );
 	if (!nrlen) nrlen = in->bv_len;
@@ -626,7 +657,8 @@ hdb_dn2id(
 	key.data = &idp;
 	key.ulen = sizeof(ID);
 	key.flags = DB_DBT_USERMEM;
-	BDB_ID2DISK( ei->bei_parent->bei_id, &idp );
+	parentID = ( ei->bei_parent != NULL ) ? ei->bei_parent->bei_id : 0;
+	BDB_ID2DISK( parentID, &idp );
 
 	DBTzero(&data);
 	data.size = sizeof(diskNode) + nrlen - sizeof(ID) - 1;
@@ -640,21 +672,24 @@ hdb_dn2id(
 	d = op->o_tmpalloc( data.size * 3, op->o_tmpmemctx );
 	d->nrdnlen[1] = nrlen & 0xff;
 	d->nrdnlen[0] = (nrlen >> 8) | 0x80;
+	dlen[0] = d->nrdnlen[0];
+	dlen[1] = d->nrdnlen[1];
 	ptr = lutil_strncopy( d->nrdn, in->bv_val, nrlen );
 	*ptr = '\0';
 	data.data = d;
 
 	rc = cursor->c_get( cursor, &key, &data, DB_GET_BOTH_RANGE );
-	if ( rc == 0 && strncmp( d->nrdn, in->bv_val, nrlen )) {
+	if ( rc == 0 && (dlen[1] != d->nrdnlen[1] || dlen[0] != d->nrdnlen[0] ||
+		strncmp( d->nrdn, in->bv_val, nrlen ))) {
 		rc = DB_NOTFOUND;
 	}
 	if ( rc == 0 ) {
-		ptr = data.data + data.size - sizeof(ID);
+		ptr = (char *) data.data + data.size - sizeof(ID);
 		BDB_DISK2ID( ptr, &ei->bei_id );
 		ei->bei_rdn.bv_len = data.size - sizeof(diskNode) - nrlen;
 		ptr = d->nrdn + nrlen + 1;
 		ber_str2bv( ptr, ei->bei_rdn.bv_len, 1, &ei->bei_rdn );
-		if ( !ei->bei_parent->bei_dkids ) {
+		if ( ei->bei_parent != NULL && !ei->bei_parent->bei_dkids ) {
 			db_recno_t dkids;
 			/* How many children does the parent have? */
 			/* FIXME: do we need to lock the parent
@@ -674,6 +709,7 @@ int
 hdb_dn2id_parent(
 	Operation *op,
 	DB_TXN *txn,
+	u_int32_t	locker,
 	EntryInfo *ei,
 	ID *idp )
 {
@@ -684,7 +720,6 @@ hdb_dn2id_parent(
 	int		rc = 0;
 	diskNode *d;
 	char	*ptr;
-	unsigned char *pt2;
 	ID	nid;
 
 	DBTzero(&key);
@@ -699,6 +734,9 @@ hdb_dn2id_parent(
 
 	rc = db->cursor( db, txn, &cursor, bdb->bi_db_opflags );
 	if ( rc ) return rc;
+	if ( !txn && locker ) {
+		cursor->locker = locker;
+	}
 
 	data.ulen = sizeof(diskNode) + (SLAP_LDAPDN_MAXLEN * 2);
 	d = op->o_tmpalloc( data.ulen, op->o_tmpmemctx );
@@ -710,7 +748,7 @@ hdb_dn2id_parent(
 			rc = LDAP_OTHER;
 		} else {
 			db_recno_t dkids;
-			ptr = data.data + data.size - sizeof(ID);
+			ptr = (char *) data.data + data.size - sizeof(ID);
 			BDB_DISK2ID( ptr, idp );
 			ei->bei_nrdn.bv_len = (d->nrdnlen[0] << 8) | d->nrdnlen[1];
 			ber_str2bv( d->nrdn, ei->bei_nrdn.bv_len, 1, &ei->bei_nrdn );
@@ -791,21 +829,22 @@ hdb_dn2id_children(
 
 struct dn2id_cookie {
 	struct bdb_info *bdb;
-	DB *db;
-	int prefix;
-	int rc;
+	Operation *op;
 	EntryInfo *ei;
-	ID id;
-	ID nid;
-	ID dbuf;
 	ID *ids;
-	void *ptr;
-	ID tmp[BDB_IDL_DB_SIZE];
+	ID *tmp;
 	ID *buf;
+	DB *db;
+	DBC *dbc;
 	DBT key;
 	DBT data;
-	DBC *dbc;
-	Operation *op;
+	ID dbuf;
+	ID id;
+	ID nid;
+	int rc;
+	int depth;
+	char need_sort;
+	char prefix;
 };
 
 static int
@@ -816,7 +855,7 @@ apply_func(
 	EntryInfo *ei = data;
 	ID *idl = arg;
 
-	bdb_idl_insert( idl, ei->bei_id );
+	bdb_idl_append_one( idl, ei->bei_id );
 	return 0;
 }
 
@@ -825,23 +864,32 @@ hdb_dn2idl_internal(
 	struct dn2id_cookie *cx
 )
 {
+	BDB_IDL_ZERO( cx->tmp );
+
 	if ( cx->bdb->bi_idl_cache_size ) {
-		cx->key.data = &cx->id;
-		cx->rc = bdb_idl_cache_get(cx->bdb, cx->db, &cx->key, cx->tmp);
-		if ( cx->rc == DB_NOTFOUND ) {
-			return cx->rc;
+		char *ptr = ((char *)&cx->id)-1;
+
+		cx->key.data = ptr;
+		cx->key.size = sizeof(ID)+1;
+		if ( cx->prefix == DN_SUBTREE_PREFIX ) {
+			ID *ids = cx->depth ? cx->tmp : cx->ids;
+			*ptr = cx->prefix;
+			cx->rc = bdb_idl_cache_get(cx->bdb, cx->db, &cx->key, ids);
+			if ( cx->rc == LDAP_SUCCESS ) {
+				if ( cx->depth ) {
+					bdb_idl_append( cx->ids, cx->tmp );
+					cx->need_sort = 1;
+				}
+				return cx->rc;
+			}
 		}
+		*ptr = DN_ONE_PREFIX;
+		cx->rc = bdb_idl_cache_get(cx->bdb, cx->db, &cx->key, cx->tmp);
 		if ( cx->rc == LDAP_SUCCESS ) {
 			goto gotit;
 		}
-	}
-	BDB_IDL_ZERO( cx->tmp );
-
-	if ( !cx->ei ) {
-		cx->ei = bdb_cache_find_info( cx->bdb, cx->id );
-		if ( !cx->ei ) {
-			cx->rc = DB_NOTFOUND;
-			goto saveit;
+		if ( cx->rc == DB_NOTFOUND ) {
+			return cx->rc;
 		}
 	}
 
@@ -855,11 +903,24 @@ hdb_dn2idl_internal(
 		db_recno_t dkids = cx->ei->bei_dkids;
 		ei.bei_parent = cx->ei;
 
+		/* Only one thread should load the cache */
+		while ( cx->ei->bei_state & CACHE_ENTRY_ONELEVEL ) {
+			bdb_cache_entryinfo_unlock( cx->ei );
+			ldap_pvt_thread_yield();
+			bdb_cache_entryinfo_lock( cx->ei );
+			if ( cx->ei->bei_ckids+1 == cx->ei->bei_dkids ) {
+				goto synced;
+			}
+		}
+
+		cx->ei->bei_state |= CACHE_ENTRY_ONELEVEL;
+
 		bdb_cache_entryinfo_unlock( cx->ei );
 
 		cx->rc = cx->db->cursor( cx->db, NULL, &cx->dbc,
 			cx->bdb->bi_db_opflags );
-		if ( cx->rc ) return cx->rc;
+		if ( cx->rc )
+			goto done_one;
 
 		cx->data.data = &cx->dbuf;
 		cx->data.ulen = sizeof(ID);
@@ -868,11 +929,11 @@ hdb_dn2idl_internal(
 
 		/* The first item holds the parent ID. Ignore it. */
 		cx->key.data = &cx->nid;
+		cx->key.size = sizeof(ID);
 		cx->rc = cx->dbc->c_get( cx->dbc, &cx->key, &cx->data, DB_SET );
 		if ( cx->rc ) {
 			cx->dbc->c_close( cx->dbc );
-			if ( cx->rc == DB_NOTFOUND ) goto saveit;
-			return cx->rc;
+			goto done_one;
 		}
 
 		/* If the on-disk count is zero we've never checked it.
@@ -887,43 +948,55 @@ hdb_dn2idl_internal(
 		cx->data.ulen = BDB_IDL_UM_SIZE * sizeof(ID);
 		cx->data.flags = DB_DBT_USERMEM;
 
-		/* Fetch the rest of the IDs in a loop... */
-		while ( (cx->rc = cx->dbc->c_get( cx->dbc, &cx->key, &cx->data,
-			DB_MULTIPLE | DB_NEXT_DUP )) == 0 ) {
-			u_int8_t *j;
-			size_t len;
-			DB_MULTIPLE_INIT( cx->ptr, &cx->data );
-			while (cx->ptr) {
-				DB_MULTIPLE_NEXT( cx->ptr, &cx->data, j, len );
-				if (j) {
-					EntryInfo *ei2;
-					diskNode *d = (diskNode *)j;
-					short nrlen;
+		if ( dkids > 1 ) {
+			/* Fetch the rest of the IDs in a loop... */
+			while ( (cx->rc = cx->dbc->c_get( cx->dbc, &cx->key, &cx->data,
+				DB_MULTIPLE | DB_NEXT_DUP )) == 0 ) {
+				u_int8_t *j;
+				size_t len;
+				void *ptr;
+				DB_MULTIPLE_INIT( ptr, &cx->data );
+				while (ptr) {
+					DB_MULTIPLE_NEXT( ptr, &cx->data, j, len );
+					if (j) {
+						EntryInfo *ei2;
+						diskNode *d = (diskNode *)j;
+						short nrlen;
 
-					BDB_DISK2ID( j + len - sizeof(ID), &ei.bei_id );
-					nrlen = ((d->nrdnlen[0] ^ 0x80) << 8) | d->nrdnlen[1];
-					ei.bei_nrdn.bv_len = nrlen;
-					/* nrdn/rdn are set in-place.
-					 * hdb_cache_load will copy them as needed
-					 */
-					ei.bei_nrdn.bv_val = d->nrdn;
-					ei.bei_rdn.bv_len = len - sizeof(diskNode)
-						- ei.bei_nrdn.bv_len;
-					ei.bei_rdn.bv_val = d->nrdn + ei.bei_nrdn.bv_len + 1;
-					bdb_idl_insert( cx->tmp, ei.bei_id );
-					hdb_cache_load( cx->bdb, &ei, &ei2 );
+						BDB_DISK2ID( j + len - sizeof(ID), &ei.bei_id );
+						nrlen = ((d->nrdnlen[0] ^ 0x80) << 8) | d->nrdnlen[1];
+						ei.bei_nrdn.bv_len = nrlen;
+						/* nrdn/rdn are set in-place.
+						 * hdb_cache_load will copy them as needed
+						 */
+						ei.bei_nrdn.bv_val = d->nrdn;
+						ei.bei_rdn.bv_len = len - sizeof(diskNode)
+							- ei.bei_nrdn.bv_len;
+						ei.bei_rdn.bv_val = d->nrdn + ei.bei_nrdn.bv_len + 1;
+						bdb_idl_append_one( cx->tmp, ei.bei_id );
+						hdb_cache_load( cx->bdb, &ei, &ei2 );
+					}
 				}
 			}
 		}
+
 		cx->rc = cx->dbc->c_close( cx->dbc );
+done_one:
+		bdb_cache_entryinfo_lock( cx->ei );
+		cx->ei->bei_state ^= CACHE_ENTRY_ONELEVEL;
+		bdb_cache_entryinfo_unlock( cx->ei );
+		if ( cx->rc )
+			return cx->rc;
+
 	} else {
 		/* The in-memory cache is in sync with the on-disk data.
 		 * do we have any kids?
 		 */
+synced:
 		cx->rc = 0;
 		if ( cx->ei->bei_ckids > 0 ) {
-			/* Walk the kids tree; order is irrelevant since bdb_idl_insert
-			 * will insert in sorted order.
+			/* Walk the kids tree; order is irrelevant since bdb_idl_sort
+			 * will sort it later.
 			 */
 			avl_apply( cx->ei->bei_kids, apply_func,
 				cx->tmp, -1, AVL_POSTORDER );
@@ -931,36 +1004,45 @@ hdb_dn2idl_internal(
 		bdb_cache_entryinfo_unlock( cx->ei );
 	}
 
-saveit:
-	if ( cx->bdb->bi_idl_cache_max_size ) {
-		cx->key.data = &cx->id;
+	if ( !BDB_IDL_IS_RANGE( cx->tmp ) && cx->tmp[0] > 3 )
+		bdb_idl_sort( cx->tmp, cx->buf );
+	if ( cx->bdb->bi_idl_cache_max_size && !BDB_IDL_IS_ZERO( cx->tmp )) {
+		char *ptr = ((char *)&cx->id)-1;
+		cx->key.data = ptr;
+		cx->key.size = sizeof(ID)+1;
+		*ptr = DN_ONE_PREFIX;
 		bdb_idl_cache_put( cx->bdb, cx->db, &cx->key, cx->tmp, cx->rc );
 	}
-	;
+
 gotit:
 	if ( !BDB_IDL_IS_ZERO( cx->tmp )) {
 		if ( cx->prefix == DN_SUBTREE_PREFIX ) {
-			if (cx->ei->bei_state & CACHE_ENTRY_NO_GRANDKIDS) {
-				bdb_idl_union( cx->ids, cx->tmp );
-			} else {
+			bdb_idl_append( cx->ids, cx->tmp );
+			cx->need_sort = 1;
+			if ( !(cx->ei->bei_state & CACHE_ENTRY_NO_GRANDKIDS)) {
 				ID *save, idcurs;
 				EntryInfo *ei = cx->ei;
 				int nokids = 1;
 				save = cx->op->o_tmpalloc( BDB_IDL_SIZEOF( cx->tmp ),
 					cx->op->o_tmpmemctx );
 				BDB_IDL_CPY( save, cx->tmp );
-				bdb_idl_union( cx->ids, cx->tmp );
 
 				idcurs = 0;
+				cx->depth++;
 				for ( cx->id = bdb_idl_first( save, &idcurs );
 					cx->id != NOID;
 					cx->id = bdb_idl_next( save, &idcurs )) {
+					cx->ei = bdb_cache_find_info( cx->bdb, cx->id );
+					if ( !cx->ei ||
+						( cx->ei->bei_state & CACHE_ENTRY_NO_KIDS ))
+						continue;
+
 					BDB_ID2DISK( cx->id, &cx->nid );
-					cx->ei = NULL;
 					hdb_dn2idl_internal( cx );
 					if ( !BDB_IDL_IS_ZERO( cx->tmp ))
 						nokids = 0;
 				}
+				cx->depth--;
 				cx->op->o_tmpfree( save, cx->op->o_tmpmemctx );
 				if ( nokids ) ei->bei_state |= CACHE_ENTRY_NO_GRANDKIDS;
 			}
@@ -1002,16 +1084,23 @@ hdb_dn2idl(
 	cx.ei = e->e_id ? BEI(e) : &bdb->bi_cache.c_dntree;
 	cx.bdb = bdb;
 	cx.db = cx.bdb->bi_dn2id->bdi_db;
-	cx.prefix = op->ors_scope == LDAP_SCOPE_ONELEVEL
-		? DN_ONE_PREFIX : DN_SUBTREE_PREFIX;
+	cx.prefix = (op->ors_scope == LDAP_SCOPE_ONELEVEL) ?
+		DN_ONE_PREFIX : DN_SUBTREE_PREFIX;
 	cx.ids = ids;
-	cx.buf = stack;
+	cx.tmp = stack;
+	cx.buf = stack + BDB_IDL_UM_SIZE;
 	cx.op = op;
+	cx.need_sort = 0;
+	cx.depth = 0;
 
-	BDB_IDL_ZERO( ids );
 	if ( cx.prefix == DN_SUBTREE_PREFIX ) {
-		bdb_idl_insert( ids, cx.id );
+		ids[0] = 1;
+		ids[1] = cx.id;
+	} else {
+		BDB_IDL_ZERO( ids );
 	}
+	if ( cx.ei->bei_state & CACHE_ENTRY_NO_KIDS )
+		return LDAP_SUCCESS;
 
 	DBTzero(&cx.key);
 	cx.key.ulen = sizeof(ID);
@@ -1020,6 +1109,21 @@ hdb_dn2idl(
 
 	DBTzero(&cx.data);
 
-	return hdb_dn2idl_internal(&cx);
+	hdb_dn2idl_internal(&cx);
+	if ( cx.need_sort ) {
+		char *ptr = ((char *)&cx.id)-1;
+		if ( !BDB_IDL_IS_RANGE( cx.ids ) && cx.ids[0] > 3 ) 
+			bdb_idl_sort( cx.ids, cx.tmp );
+		cx.key.data = ptr;
+		cx.key.size = sizeof(ID)+1;
+		*ptr = cx.prefix;
+		cx.id = e->e_id;
+		bdb_idl_cache_put( cx.bdb, cx.db, &cx.key, cx.ids, cx.rc );
+	}
+
+	if ( cx.rc == DB_NOTFOUND )
+		cx.rc = LDAP_SUCCESS;
+
+	return cx.rc;
 }
 #endif	/* BDB_HIER */
