@@ -1,8 +1,8 @@
 /* config.c - DNS SRV backend configuration file routine */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-dnssrv/config.c,v 1.13.2.1 2005/01/20 18:03:59 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-dnssrv/config.c,v 1.16 2006/01/03 22:12:18 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2005 The OpenLDAP Foundation.
+ * Copyright 2000-2006 The OpenLDAP Foundation.
  * Portions Copyright 2000-2003 Kurt D. Zeilenga.
  * All rights reserved.
  *
@@ -37,6 +37,7 @@ dnssrv_back_db_config(
     int		argc,
     char	**argv )
 {
+#if 0
 	struct ldapinfo	*li = (struct ldapinfo *) be->be_private;
 
 	if ( li == NULL ) {
@@ -44,6 +45,7 @@ dnssrv_back_db_config(
 		    fname, lineno );
 		return( 1 );
 	}
+#endif
 
 	/* no configuration options (yet) */
 	return SLAP_CONF_UNKNOWN;
