@@ -1,8 +1,8 @@
 /* bind.c - monitor backend bind routine */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/bind.c,v 1.14.2.1 2005/01/20 18:04:00 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/bind.c,v 1.17 2006/01/03 22:12:21 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001-2005 The OpenLDAP Foundation.
+ * Copyright 2001-2006 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -33,23 +33,19 @@
 int
 monitor_back_bind( Operation *op, SlapReply *rs )
 {
-#if 0	/* not used yet */
-	monitor_info_t	*mi
-		= (monitor_info_t *) op->o_bd->be_private;
-#endif
-
 	Debug(LDAP_DEBUG_ARGS, "==> monitor_back_bind: dn: %s\n", 
 			op->o_req_dn.bv_val, 0, 0 );
 	
 	if ( op->oq_bind.rb_method == LDAP_AUTH_SIMPLE 
-			&& be_isroot_pw( op ) ) {
+			&& be_isroot_pw( op ) )
+	{
 		ber_dupbv( &op->oq_bind.rb_edn, be_root_dn( op->o_bd ) );
-		return( 0 );
+		return LDAP_SUCCESS;
 	}
 
 	rs->sr_err = LDAP_INVALID_CREDENTIALS;
 	send_ldap_result( op, rs );
 
-	return( 1 );
+	return rs->sr_err;
 }
 
