@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/back-sql.h,v 1.30.2.2 2005/01/24 21:28:53 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/back-sql.h,v 1.49 2006/01/03 22:12:24 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2005 The OpenLDAP Foundation.
+ * Copyright 1999-2006 The OpenLDAP Foundation.
  * Portions Copyright 1999 Dmitry Kovalev.
  * Portions Copyright 2002 Pierangelo Mararati.
  * Portions Copyright 2004 Mark Adamson.
@@ -223,11 +223,9 @@ typedef struct {
 #undef BACKSQL_ARBITRARY_KEY
 
 /*
- * define to enable experimental support for syncporv overlay
+ * define to enable support for syncprov overlay
  */
-#ifdef LDAP_DEVEL
 #define BACKSQL_SYNCPROV
-#endif /* LDAP_DEVEL */
 
 /*
  * define to the appropriate aliasing string
@@ -436,15 +434,12 @@ typedef struct backsql_srch_info {
  * is used */
 #define BACKSQL_SCOPE_BASE_LIKE		( LDAP_SCOPE_BASE | 0x1000 )
 	Filter			*bsi_filter;
-	int			bsi_slimit,
-				bsi_tlimit;
 	time_t			bsi_stoptime;
 
 	backsql_entryID		*bsi_id_list,
 				**bsi_id_listtail,
 				*bsi_c_eid;
 	int			bsi_n_candidates;
-	int			bsi_abandon;
 	int			bsi_status;
 
 	backsql_oc_map_rec	*bsi_oc;
@@ -494,6 +489,10 @@ typedef struct backsql_info {
 	BerVarray	sql_concat_func;
 	struct berval	sql_strcast_func;
 
+	struct berval	sql_aliasing;
+	struct berval	sql_aliasing_quote;
+	struct berval	sql_dn_oc_aliasing;
+
 	AttributeName	*sql_anlist;
 
 	unsigned int	sql_flags;
@@ -509,6 +508,7 @@ typedef struct backsql_info {
 #define BSQLF_FETCH_ALL_USERATTRS	0x0200
 #define BSQLF_FETCH_ALL_OPATTRS		0x0400
 #define	BSQLF_FETCH_ALL_ATTRS		(BSQLF_FETCH_ALL_USERATTRS|BSQLF_FETCH_ALL_OPATTRS)
+#define BSQLF_CHECK_SCHEMA		0x0800
 
 #define BACKSQL_ISF(si, f) \
 	(((si)->sql_flags & f) == f)
@@ -539,6 +539,8 @@ typedef struct backsql_info {
 	BACKSQL_ISF(si, BSQLF_FETCH_ALL_OPATTRS)
 #define BACKSQL_FETCH_ALL_ATTRS(si) \
 	BACKSQL_ISF(si, BSQLF_FETCH_ALL_ATTRS)
+#define BACKSQL_CHECK_SCHEMA(si) \
+	BACKSQL_ISF(si, BSQLF_CHECK_SCHEMA)
 
 	Entry		*sql_baseObject;
 #ifdef BACKSQL_ARBITRARY_KEY
