@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/modrdn.c,v 1.14.2.1 2005/01/20 18:04:02 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/modrdn.c,v 1.39.2.1 2006/04/18 17:33:11 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2005 The OpenLDAP Foundation.
+ * Copyright 1999-2006 The OpenLDAP Foundation.
  * Portions Copyright 1999 Dmitry Kovalev.
  * Portions Copyright 2002 Pierangelo Masarati.
  * All rights reserved.
@@ -44,14 +44,11 @@ backsql_modrdn( Operation *op, SlapReply *rs )
 				*new_pdn = NULL, *new_npdn = NULL,
 				new_dn = BER_BVNULL, new_ndn = BER_BVNULL,
 				realnew_dn = BER_BVNULL;
-	LDAPRDN			new_rdn = NULL;
-	LDAPRDN			old_rdn = NULL;
 	Entry			r = { 0 },
 				p = { 0 },
 				n = { 0 },
 				*e = NULL;
 	int			manageDSAit = get_manageDSAit( op );
-	Modifications		*mod = NULL;
 	struct berval		*newSuperior = op->oq_modrdn.rs_newSup;
 	char			*next;
  
@@ -74,7 +71,6 @@ backsql_modrdn( Operation *op, SlapReply *rs )
 	bsi.bsi_e = &r;
 	rs->sr_err = backsql_init_search( &bsi, &op->o_req_ndn,
 			LDAP_SCOPE_BASE, 
-			SLAP_NO_LIMIT, SLAP_NO_LIMIT,
 			(time_t)(-1), NULL, dbh, op, rs,
 			slap_anlist_all_attributes,
 			( BACKSQL_ISF_MATCHED | BACKSQL_ISF_GET_ENTRY ) );
@@ -129,7 +125,7 @@ backsql_modrdn( Operation *op, SlapReply *rs )
 		goto done;
 	}
 
-	if ( backsql_has_children( bi, dbh, &op->o_req_ndn ) == LDAP_COMPARE_TRUE ) {
+	if ( backsql_has_children( op, dbh, &op->o_req_ndn ) == LDAP_COMPARE_TRUE ) {
 		Debug( LDAP_DEBUG_TRACE, "   backsql_modrdn(): "
 			"entry \"%s\" has children\n",
 			op->o_req_dn.bv_val, 0, 0 );
@@ -171,7 +167,6 @@ backsql_modrdn( Operation *op, SlapReply *rs )
 	e_id = bsi.bsi_base_id;
 	rs->sr_err = backsql_init_search( &bsi, &pndn,
 			LDAP_SCOPE_BASE, 
-			SLAP_NO_LIMIT, SLAP_NO_LIMIT,
 			(time_t)(-1), NULL, dbh, op, rs,
 			slap_anlist_all_attributes,
 			BACKSQL_ISF_GET_ENTRY );
@@ -194,8 +189,9 @@ backsql_modrdn( Operation *op, SlapReply *rs )
 		goto done;
 	}
 
-	if ( !access_allowed( op, &p, slap_schema.si_ad_children, 
-				NULL, ACL_WRITE, NULL ) ) {
+	if ( !access_allowed( op, &p, slap_schema.si_ad_children, NULL,
+			newSuperior ? ACL_WDEL : ACL_WRITE, NULL ) )
+	{
 		Debug( LDAP_DEBUG_TRACE, "   no access to parent\n", 0, 0, 0 );
 		rs->sr_err = LDAP_INSUFFICIENT_ACCESS;
 		goto done;
@@ -225,7 +221,6 @@ backsql_modrdn( Operation *op, SlapReply *rs )
 		bsi.bsi_e = &n;
 		rs->sr_err = backsql_init_search( &bsi, new_npdn,
 				LDAP_SCOPE_BASE, 
-				SLAP_NO_LIMIT, SLAP_NO_LIMIT,
 				(time_t)(-1), NULL, dbh, op, rs,
 				slap_anlist_all_attributes,
 				( BACKSQL_ISF_MATCHED | BACKSQL_ISF_GET_ENTRY ) );
@@ -250,7 +245,7 @@ backsql_modrdn( Operation *op, SlapReply *rs )
 #endif /* ! BACKSQL_ARBITRARY_KEY */
 
 		if ( !access_allowed( op, &n, slap_schema.si_ad_children, 
-					NULL, ACL_WRITE, NULL ) ) {
+					NULL, ACL_WADD, NULL ) ) {
 			Debug( LDAP_DEBUG_TRACE, "   backsql_modrdn(): "
 					"no access to new parent \"%s\"\n", 
 					new_pdn->bv_val, 0, 0 );
@@ -397,64 +392,27 @@ backsql_modrdn( Operation *op, SlapReply *rs )
 	}
 	SQLFreeStmt( sth, SQL_DROP );
 
-	/*
-	 * Get attribute type and attribute value of our new rdn,
-	 * we will need to add that to our new entry
-	 */
-	if ( ldap_bv2rdn( &op->oq_modrdn.rs_newrdn, &new_rdn, &next, 
-				LDAP_DN_FORMAT_LDAP ) )
-	{
-		Debug( LDAP_DEBUG_TRACE,
-			"   backsql_modrdn: can't figure out "
-			"type(s)/values(s) of new_rdn\n", 
-			0, 0, 0 );
-		rs->sr_err = LDAP_INVALID_DN_SYNTAX;
-		e = &r;
-		goto done;
-	}
+	assert( op->orr_modlist != NULL );
 
-	Debug( LDAP_DEBUG_TRACE, "backsql_modrdn: "
-		"new_rdn_type=\"%s\", new_rdn_val=\"%s\"\n",
-		new_rdn[ 0 ]->la_attr.bv_val,
-		new_rdn[ 0 ]->la_value.bv_val, 0 );
-
-	if ( op->oq_modrdn.rs_deleteoldrdn ) {
-		if ( ldap_bv2rdn( &op->o_req_dn, &old_rdn, &next,
-					LDAP_DN_FORMAT_LDAP ) )
-		{
-			Debug( LDAP_DEBUG_TRACE,
-				"   backsql_modrdn: can't figure out "
-				"the old_rdn type(s)/value(s)\n", 
-				0, 0, 0 );
-			rs->sr_err = LDAP_OTHER;
-			e = NULL;
-			goto done;
-		}
-	}
-
-	rs->sr_err = slap_modrdn2mods( op, rs, &r, old_rdn, new_rdn, &mod );
-	if ( rs->sr_err != LDAP_SUCCESS ) {
-		e = &r;
-		goto done;
-	}
+	slap_mods_opattrs( op, &op->orr_modlist, 1 );
 
 	oc = backsql_id2oc( bi, e_id.eid_oc_id );
-	rs->sr_err = backsql_modify_internal( op, rs, dbh, oc, &e_id, mod );
+	rs->sr_err = backsql_modify_internal( op, rs, dbh, oc, &e_id, op->orr_modlist );
+	slap_graduate_commit_csn( op );
 	if ( rs->sr_err != LDAP_SUCCESS ) {
 		e = &r;
 		goto done;
 	}
 
-	if ( global_schemacheck ) {
+	if ( BACKSQL_CHECK_SCHEMA( bi ) ) {
 		char		textbuf[ SLAP_TEXT_BUFLEN ] = { '\0' };
 
-		entry_clean( &r );
+		backsql_entry_clean( op, &r );
 		(void)backsql_free_entryID( op, &e_id, 0 );
 
 		bsi.bsi_e = &r;
 		rs->sr_err = backsql_init_search( &bsi, &new_ndn,
 				LDAP_SCOPE_BASE, 
-				SLAP_NO_LIMIT, SLAP_NO_LIMIT,
 				(time_t)(-1), NULL, dbh, op, rs,
 				slap_anlist_all_attributes,
 				( BACKSQL_ISF_MATCHED | BACKSQL_ISF_GET_ENTRY ) );
@@ -494,9 +452,8 @@ backsql_modrdn( Operation *op, SlapReply *rs )
 
 		e_id = bsi.bsi_base_id;
 
-		rs->sr_err = entry_schema_check( op->o_bd, &r,
-				NULL,
-				&rs->sr_text, textbuf, sizeof( textbuf ) );
+		rs->sr_err = entry_schema_check( op, &r, NULL, 0,
+			&rs->sr_text, textbuf, sizeof( textbuf ) );
 		if ( rs->sr_err != LDAP_SUCCESS ) {
 			Debug( LDAP_DEBUG_TRACE, "   backsql_add(\"%s\"): "
 				"entry failed schema check -- aborting\n",
@@ -507,7 +464,6 @@ backsql_modrdn( Operation *op, SlapReply *rs )
 	}
 
 done:;
-#ifdef SLAP_ACL_HONOR_DISCLOSE
 	if ( e != NULL ) {
 		if ( !access_allowed( op, e, slap_schema.si_ad_entry, NULL,
 					ACL_DISCLOSE, NULL ) )
@@ -521,9 +477,6 @@ done:;
 			}
 		}
 	}
-#endif /* SLAP_ACL_HONOR_DISCLOSE */
-
-	send_ldap_result( op, rs );
 
 	/*
 	 * Commit only if all operations succeed
@@ -538,6 +491,9 @@ done:;
 		SQLTransact( SQL_NULL_HENV, dbh, CompletionType );
 	}
 
+	send_ldap_result( op, rs );
+	slap_graduate_commit_csn( op );
+
 	if ( !BER_BVISNULL( &realnew_dn ) && realnew_dn.bv_val != new_dn.bv_val ) {
 		ch_free( realnew_dn.bv_val );
 	}
@@ -550,21 +506,6 @@ done:;
 		slap_sl_free( new_ndn.bv_val, op->o_tmpmemctx );
 	}
 	
-	/* LDAP v2 supporting correct attribute handling. */
-	if ( new_rdn != NULL ) {
-		ldap_rdnfree( new_rdn );
-	}
-	if ( old_rdn != NULL ) {
-		ldap_rdnfree( old_rdn );
-	}
-	if ( mod != NULL ) {
-		Modifications *tmp;
-		for (; mod; mod = tmp ) {
-			tmp = mod->sml_next;
-			free( mod );
-		}
-	}
-
 	if ( !BER_BVISNULL( &e_id.eid_ndn ) ) {
 		(void)backsql_free_entryID( op, &e_id, 0 );
 	}
@@ -574,15 +515,20 @@ done:;
 	}
 
 	if ( !BER_BVISNULL( &r.e_nname ) ) {
-		entry_clean( &r );
+		backsql_entry_clean( op, &r );
 	}
 
 	if ( !BER_BVISNULL( &p.e_nname ) ) {
-		entry_clean( &p );
+		backsql_entry_clean( op, &p );
 	}
 
 	if ( !BER_BVISNULL( &n.e_nname ) ) {
-		entry_clean( &n );
+		backsql_entry_clean( op, &n );
+	}
+
+	if ( rs->sr_ref ) {
+		ber_bvarray_free( rs->sr_ref );
+		rs->sr_ref = NULL;
 	}
 
 	Debug( LDAP_DEBUG_TRACE, "<==backsql_modrdn()\n", 0, 0, 0 );
