@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/search.c,v 1.81.2.2 2005/01/24 21:28:53 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/search.c,v 1.117.2.1 2006/04/18 17:33:11 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2005 The OpenLDAP Foundation.
+ * Copyright 1999-2006 The OpenLDAP Foundation.
  * Portions Copyright 1999 Dmitry Kovalev.
  * Portions Copyright 2002 Pierangelo Masarati.
  * Portions Copyright 2004 Mark Adamson.
@@ -28,6 +28,7 @@
 #include "ac/string.h"
 #include "ac/ctype.h"
 
+#include "lutil.h"
 #include "slap.h"
 #include "proto-sql.h"
 
@@ -109,8 +110,6 @@ backsql_init_search(
 	backsql_srch_info 	*bsi, 
 	struct berval		*nbase, 
 	int 			scope, 
-	int 			slimit,
-	int 			tlimit,
 	time_t 			stoptime, 
 	Filter 			*filter, 
 	SQLHDBC 		dbh,
@@ -127,8 +126,6 @@ backsql_init_search(
 	BER_BVZERO( &bsi->bsi_base_id.eid_dn );
 	BER_BVZERO( &bsi->bsi_base_id.eid_ndn );
 	bsi->bsi_scope = scope;
-	bsi->bsi_slimit = slimit;
-	bsi->bsi_tlimit = tlimit;
 	bsi->bsi_filter = filter;
 	bsi->bsi_dbh = dbh;
 	bsi->bsi_op = op;
@@ -253,7 +250,6 @@ backsql_init_search(
 		}
 	}
 
-	bsi->bsi_abandon = 0;
 	bsi->bsi_id_list = NULL;
 	bsi->bsi_id_listtail = &bsi->bsi_id_list;
 	bsi->bsi_n_candidates = 0;
@@ -273,7 +269,7 @@ backsql_init_search(
 		int	getentry = BACKSQL_IS_GET_ENTRY( flags );
 		int	gotit = 0;
 
-		assert( op->o_bd->be_private );
+		assert( op->o_bd->be_private != NULL );
 
 		rc = backsql_dn2id( op, rs, dbh, nbase, &bsi->bsi_base_id,
 				matched, 1 );
@@ -350,7 +346,8 @@ backsql_process_filter_list( backsql_srch_info *bsi, Filter *f, int op )
 		return 0;
 	}
 
-	backsql_strfcat( &bsi->bsi_flt_where, "c", '(' /* ) */  );
+	backsql_strfcat_x( &bsi->bsi_flt_where,
+			bsi->bsi_op->o_tmpmemctx, "c", '(' /* ) */  );
 
 	while ( 1 ) {
 		res = backsql_process_filter( bsi, f );
@@ -369,20 +366,23 @@ backsql_process_filter_list( backsql_srch_info *bsi, Filter *f, int op )
 
 		switch ( op ) {
 		case LDAP_FILTER_AND:
-			backsql_strfcat( &bsi->bsi_flt_where, "l",
+			backsql_strfcat_x( &bsi->bsi_flt_where,
+					bsi->bsi_op->o_tmpmemctx, "l",
 					(ber_len_t)STRLENOF( " AND " ), 
 						" AND " );
 			break;
 
 		case LDAP_FILTER_OR:
-			backsql_strfcat( &bsi->bsi_flt_where, "l",
+			backsql_strfcat_x( &bsi->bsi_flt_where,
+					bsi->bsi_op->o_tmpmemctx, "l",
 					(ber_len_t)STRLENOF( " OR " ),
 						" OR " );
 			break;
 		}
 	}
 
-	backsql_strfcat( &bsi->bsi_flt_where, "c", /* ( */ ')' );
+	backsql_strfcat_x( &bsi->bsi_flt_where,
+			bsi->bsi_op->o_tmpmemctx, "c", /* ( */ ')' );
 
 	return 1;
 }
@@ -487,7 +487,8 @@ backsql_process_sub_filter( backsql_srch_info *bsi, Filter *f,
 	 * SQL filters are more liberal.
 	 */
 
-	backsql_strfcat( &bsi->bsi_flt_where, "c", '(' /* ) */  );
+	backsql_strfcat_x( &bsi->bsi_flt_where,
+			bsi->bsi_op->o_tmpmemctx, "c", '(' /* ) */  );
 
 	/* TimesTen */
 	Debug( LDAP_DEBUG_TRACE, "backsql_process_sub_filter(%s):\n",
@@ -500,14 +501,17 @@ backsql_process_sub_filter( backsql_srch_info *bsi, Filter *f,
 		 * If a pre-upper-cased version of the column 
 		 * or a precompiled upper function exists, use it
 		 */
-		backsql_strfcat( &bsi->bsi_flt_where, 
+		backsql_strfcat_x( &bsi->bsi_flt_where, 
+				bsi->bsi_op->o_tmpmemctx,
 				"bl",
 				&at->bam_sel_expr_u,
 				(ber_len_t)STRLENOF( " LIKE '" ),
 					" LIKE '" );
 
 	} else {
-		backsql_strfcat( &bsi->bsi_flt_where, "bl",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"bl",
 				&at->bam_sel_expr,
 				(ber_len_t)STRLENOF( " LIKE '" ), " LIKE '" );
 	}
@@ -523,14 +527,18 @@ backsql_process_sub_filter( backsql_srch_info *bsi, Filter *f,
 #endif /* BACKSQL_TRACE */
 
 		start = bsi->bsi_flt_where.bb_val.bv_len;
-		backsql_strfcat( &bsi->bsi_flt_where, "b",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"b",
 				&f->f_sub_initial );
 		if ( casefold && BACKSQL_AT_CANUPPERCASE( at ) ) {
 			ldap_pvt_str2upper( &bsi->bsi_flt_where.bb_val.bv_val[ start ] );
 		}
 	}
 
-	backsql_strfcat( &bsi->bsi_flt_where, "c", '%' );
+	backsql_strfcat_x( &bsi->bsi_flt_where,
+			bsi->bsi_op->o_tmpmemctx,
+			"c", '%' );
 
 	if ( f->f_sub_any != NULL ) {
 		for ( i = 0; !BER_BVISNULL( &f->f_sub_any[ i ] ); i++ ) {
@@ -544,7 +552,8 @@ backsql_process_sub_filter( backsql_srch_info *bsi, Filter *f,
 #endif /* BACKSQL_TRACE */
 
 			start = bsi->bsi_flt_where.bb_val.bv_len;
-			backsql_strfcat( &bsi->bsi_flt_where,
+			backsql_strfcat_x( &bsi->bsi_flt_where,
+					bsi->bsi_op->o_tmpmemctx,
 					"bc",
 					&f->f_sub_any[ i ],
 					'%' );
@@ -568,14 +577,18 @@ backsql_process_sub_filter( backsql_srch_info *bsi, Filter *f,
 #endif /* BACKSQL_TRACE */
 
 		start = bsi->bsi_flt_where.bb_val.bv_len;
-    		backsql_strfcat( &bsi->bsi_flt_where, "b",
+    		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"b",
 				&f->f_sub_final );
   		if ( casefold && BACKSQL_AT_CANUPPERCASE( at ) ) {
 			ldap_pvt_str2upper( &bsi->bsi_flt_where.bb_val.bv_val[ start ] );
 		}
 	}
 
-	backsql_strfcat( &bsi->bsi_flt_where, "l", 
+	backsql_strfcat_x( &bsi->bsi_flt_where,
+			bsi->bsi_op->o_tmpmemctx,
+			"l", 
 			(ber_len_t)STRLENOF( /* (' */ "')" ), /* (' */ "')" );
  
 	return 1;
@@ -589,18 +602,21 @@ backsql_merge_from_tbls( backsql_srch_info *bsi, struct berval *from_tbls )
 	}
 
 	if ( !BER_BVISNULL( &bsi->bsi_from.bb_val ) ) {
-		char	*start, *end, *tmp;
+		char		*start, *end;
+		struct berval	tmp;
 
-		tmp = ch_strdup( from_tbls->bv_val );
+		ber_dupbv_x( &tmp, from_tbls, bsi->bsi_op->o_tmpmemctx );
 
-		for ( start = tmp, end = strchr( start, ',' ); start; ) {
+		for ( start = tmp.bv_val, end = strchr( start, ',' ); start; ) {
 			if ( end ) {
 				end[0] = '\0';
 			}
 
 			if ( strstr( bsi->bsi_from.bb_val.bv_val, start) == NULL )
 			{
-				backsql_strfcat( &bsi->bsi_from, "cs", ',', start );
+				backsql_strfcat_x( &bsi->bsi_from,
+						bsi->bsi_op->o_tmpmemctx,
+						"cs", ',', start );
 			}
 
 			if ( end ) {
@@ -616,10 +632,12 @@ backsql_merge_from_tbls( backsql_srch_info *bsi, struct berval *from_tbls )
 			}
 		}
 
-		ch_free( tmp );
+		bsi->bsi_op->o_tmpfree( tmp.bv_val, bsi->bsi_op->o_tmpmemctx );
 
 	} else {
-		backsql_strfcat( &bsi->bsi_from, "b", from_tbls );
+		backsql_strfcat_x( &bsi->bsi_from,
+				bsi->bsi_op->o_tmpmemctx,
+				"b", from_tbls );
 	}
 
 	return LDAP_SUCCESS;
@@ -656,11 +674,15 @@ backsql_process_filter( backsql_srch_info *bsi, Filter *f )
 		break;
 
 	case LDAP_FILTER_NOT:
-		backsql_strfcat( &bsi->bsi_flt_where, "l",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"l",
 				(ber_len_t)STRLENOF( "NOT (" /* ) */ ),
 					"NOT (" /* ) */ );
 		rc = backsql_process_filter( bsi, f->f_not );
-		backsql_strfcat( &bsi->bsi_flt_where, "c", /* ( */ ')' );
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"c", /* ( */ ')' );
 		done = 1;
 		break;
 
@@ -679,7 +701,9 @@ backsql_process_filter( backsql_srch_info *bsi, Filter *f )
 			 * ldap_entries AS attributeName WHERE attributeName.dn
 			 * like '%attributeName=value%'"
 			 */
-			backsql_strfcat( &bsi->bsi_flt_where, "l",
+			backsql_strfcat_x( &bsi->bsi_flt_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"l",
 					(ber_len_t)STRLENOF( "1=1" ), "1=1" );
 			bsi->bsi_status = LDAP_SUCCESS;
 			rc = 1;
@@ -739,7 +763,9 @@ backsql_process_filter( backsql_srch_info *bsi, Filter *f )
 
 				backsql_merge_from_tbls( bsi, &ldap_entry_objclasses );
 
-				backsql_strfcat( &bsi->bsi_flt_where, "lbl",
+				backsql_strfcat_x( &bsi->bsi_flt_where,
+						bsi->bsi_op->o_tmpmemctx,
+						"lbl",
 						(ber_len_t)STRLENOF( "(2=2 OR (ldap_entries.id=ldap_entry_objclasses.entry_id AND ldap_entry_objclasses.oc_name='" /* ')) */ ),
 							"(2=2 OR (ldap_entries.id=ldap_entry_objclasses.entry_id AND ldap_entry_objclasses.oc_name='" /* ')) */,
 						&bsi->bsi_oc->bom_oc->soc_cname,
@@ -754,7 +780,9 @@ backsql_process_filter( backsql_srch_info *bsi, Filter *f )
 		}
 
 		case LDAP_FILTER_PRESENT:
-			backsql_strfcat( &bsi->bsi_flt_where, "l",
+			backsql_strfcat_x( &bsi->bsi_flt_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"l",
 					(ber_len_t)STRLENOF( "3=3" ), "3=3" );
 			bsi->bsi_status = LDAP_SUCCESS;
 			rc = 1;
@@ -793,21 +821,27 @@ backsql_process_filter( backsql_srch_info *bsi, Filter *f )
 			}
 
 #ifdef BACKSQL_ARBITRARY_KEY
-			backsql_strfcat( &bsi->bsi_flt_where, "bcblbc",
+			backsql_strfcat_x( &bsi->bsi_flt_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"bcblbc",
 					&bsi->bsi_oc->bom_keytbl, '.',
 					&bsi->bsi_oc->bom_keycol,
 					STRLENOF( " LIKE '" ), " LIKE '",
 					&keyval, '\'' );
 #else /* ! BACKSQL_ARBITRARY_KEY */
 			snprintf( keyvalbuf, sizeof( keyvalbuf ), "%lu", keyval );
-			backsql_strfcat( &bsi->bsi_flt_where, "bcbcs",
+			backsql_strfcat_x( &bsi->bsi_flt_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"bcbcs",
 					&bsi->bsi_oc->bom_keytbl, '.',
 					&bsi->bsi_oc->bom_keycol, '=', keyvalbuf );
 #endif /* ! BACKSQL_ARBITRARY_KEY */
 			break;
 
 		case LDAP_FILTER_PRESENT:
-			backsql_strfcat( &bsi->bsi_flt_where, "l",
+			backsql_strfcat_x( &bsi->bsi_flt_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"l",
 					(ber_len_t)STRLENOF( "4=4" ), "4=4" );
 			break;
 
@@ -825,18 +859,22 @@ backsql_process_filter( backsql_srch_info *bsi, Filter *f )
 		/*
 		 * support for syncrepl as producer...
 		 */
+#if 0
 		if ( !bsi->bsi_op->o_sync ) {
 			/* unsupported at present... */
 			bsi->bsi_status = LDAP_OTHER;
 			rc = -1;
 			goto done;
 		}
+#endif
 
 		bsi->bsi_flags |= ( BSQL_SF_FILTER_ENTRYCSN | BSQL_SF_RETURN_ENTRYUUID);
 
 		/* if doing a syncrepl, try to return as much as possible,
 		 * and always match the filter */
-		backsql_strfcat( &bsi->bsi_flt_where, "l",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"l",
 				(ber_len_t)STRLENOF( "5=5" ), "5=5" );
 
 		/* save for later use in operational attributes */
@@ -870,7 +908,9 @@ backsql_process_filter( backsql_srch_info *bsi, Filter *f )
 		 * selecting if there are descendants of the
 		 * candidate.
 		 */
-		backsql_strfcat( &bsi->bsi_flt_where, "l",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"l",
 				(ber_len_t)STRLENOF( "6=6" ), "6=6" );
 		if ( ad == slap_schema.si_ad_hasSubordinates ) {
 			/*
@@ -903,7 +943,9 @@ backsql_process_filter( backsql_srch_info *bsi, Filter *f )
 	if ( vat == NULL ) {
 		/* search anyway; other parts of the filter
 		 * may succeeed */
-		backsql_strfcat( &bsi->bsi_flt_where, "l",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"l",
 				(ber_len_t)STRLENOF( "7=7" ), "7=7" );
 		bsi->bsi_status = LDAP_SUCCESS;
 		rc = 1;
@@ -913,7 +955,9 @@ backsql_process_filter( backsql_srch_info *bsi, Filter *f )
 	/* if required, open extra level of parens */
 	done = 0;
 	if ( vat[0]->bam_next || vat[1] ) {
-		backsql_strfcat( &bsi->bsi_flt_where, "c", '(' );
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"c", '(' );
 		done = 1;
 	}
 
@@ -926,7 +970,9 @@ next:;
 
 	/* if more definitions of the same attr, apply */
 	if ( vat[i]->bam_next ) {
-		backsql_strfcat( &bsi->bsi_flt_where, "l",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"l",
 			STRLENOF( " OR " ), " OR " );
 		vat[i] = vat[i]->bam_next;
 		goto next;
@@ -935,14 +981,18 @@ next:;
 	/* if more descendants of the same attr, apply */
 	i++;
 	if ( vat[i] ) {
-		backsql_strfcat( &bsi->bsi_flt_where, "l",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"l",
 			STRLENOF( " OR " ), " OR " );
 		goto next;
 	}
 
 	/* if needed, close extra level of parens */
 	if ( done ) {
-		backsql_strfcat( &bsi->bsi_flt_where, "c", ')' );
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"c", ')' );
 	}
 
 	rc = 1;
@@ -972,7 +1022,9 @@ backsql_process_filter_eq( backsql_srch_info *bsi, backsql_at_map_rec *at,
 	if ( casefold && BACKSQL_AT_CANUPPERCASE( at ) ) {
 		ber_len_t	start;
 
-		backsql_strfcat( &bsi->bsi_flt_where, "cbl",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"cbl",
 				'(', /* ) */
 				&at->bam_sel_expr_u, 
 				(ber_len_t)STRLENOF( "='" ),
@@ -980,7 +1032,9 @@ backsql_process_filter_eq( backsql_srch_info *bsi, backsql_at_map_rec *at,
 
 		start = bsi->bsi_flt_where.bb_val.bv_len;
 
-		backsql_strfcat( &bsi->bsi_flt_where, "bl",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"bl",
 				filter_value, 
 				(ber_len_t)STRLENOF( /* (' */ "')" ),
 					/* (' */ "')" );
@@ -988,7 +1042,9 @@ backsql_process_filter_eq( backsql_srch_info *bsi, backsql_at_map_rec *at,
 		ldap_pvt_str2upper( &bsi->bsi_flt_where.bb_val.bv_val[ start ] );
 
 	} else {
-		backsql_strfcat( &bsi->bsi_flt_where, "cblbl",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"cblbl",
 				'(', /* ) */
 				&at->bam_sel_expr,
 				(ber_len_t)STRLENOF( "='" ), "='",
@@ -1013,7 +1069,9 @@ backsql_process_filter_like( backsql_srch_info *bsi, backsql_at_map_rec *at,
 	if ( casefold && BACKSQL_AT_CANUPPERCASE( at ) ) {
 		ber_len_t	start;
 
-		backsql_strfcat( &bsi->bsi_flt_where, "cbl",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"cbl",
 				'(', /* ) */
 				&at->bam_sel_expr_u, 
 				(ber_len_t)STRLENOF( " LIKE '%" ),
@@ -1021,7 +1079,9 @@ backsql_process_filter_like( backsql_srch_info *bsi, backsql_at_map_rec *at,
 
 		start = bsi->bsi_flt_where.bb_val.bv_len;
 
-		backsql_strfcat( &bsi->bsi_flt_where, "bl",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"bl",
 				filter_value, 
 				(ber_len_t)STRLENOF( /* (' */ "%')" ),
 					/* (' */ "%')" );
@@ -1029,7 +1089,9 @@ backsql_process_filter_like( backsql_srch_info *bsi, backsql_at_map_rec *at,
 		ldap_pvt_str2upper( &bsi->bsi_flt_where.bb_val.bv_val[ start ] );
 
 	} else {
-		backsql_strfcat( &bsi->bsi_flt_where, "cblbl",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"cblbl",
 				'(', /* ) */
 				&at->bam_sel_expr,
 				(ber_len_t)STRLENOF( " LIKE '%" ),
@@ -1066,7 +1128,9 @@ backsql_process_filter_attr( backsql_srch_info *bsi, Filter *f, backsql_at_map_r
 			&& strstr( bsi->bsi_join_where.bb_val.bv_val,
 				at->bam_join_where.bv_val ) == NULL )
 	{
-	       	backsql_strfcat( &bsi->bsi_join_where, "lb",
+	       	backsql_strfcat_x( &bsi->bsi_join_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"lb",
 				(ber_len_t)STRLENOF( " AND " ), " AND ",
 				&at->bam_join_where );
 	}
@@ -1136,7 +1200,9 @@ equality_match:;
 		if ( at->bam_ad == slap_schema.si_ad_objectClass
 				|| at->bam_ad == slap_schema.si_ad_structuralObjectClass )
 		{
-			backsql_strfcat( &bsi->bsi_flt_where, "lbl",
+			backsql_strfcat_x( &bsi->bsi_flt_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"lbl",
 					(ber_len_t)STRLENOF( "(ldap_entries.id=ldap_entry_objclasses.entry_id AND ldap_entry_objclasses.oc_name='" /* ') */ ),
 						"(ldap_entries.id=ldap_entry_objclasses.entry_id AND ldap_entry_objclasses.oc_name='" /* ') */,
 					filter_value,
@@ -1178,7 +1244,9 @@ equality_match:;
 		if ( casefold && BACKSQL_AT_CANUPPERCASE( at ) ) {
 			ber_len_t	start;
 
-			backsql_strfcat( &bsi->bsi_flt_where, "cbbc",
+			backsql_strfcat_x( &bsi->bsi_flt_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"cbbc",
 					'(', /* ) */
 					&at->bam_sel_expr_u, 
 					&ordering,
@@ -1186,7 +1254,9 @@ equality_match:;
 
 			start = bsi->bsi_flt_where.bb_val.bv_len;
 
-			backsql_strfcat( &bsi->bsi_flt_where, "bl",
+			backsql_strfcat_x( &bsi->bsi_flt_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"bl",
 					filter_value, 
 					(ber_len_t)STRLENOF( /* (' */ "')" ),
 						/* (' */ "')" );
@@ -1194,7 +1264,9 @@ equality_match:;
 			ldap_pvt_str2upper( &bsi->bsi_flt_where.bb_val.bv_val[ start ] );
 		
 		} else {
-			backsql_strfcat( &bsi->bsi_flt_where, "cbbcbl",
+			backsql_strfcat_x( &bsi->bsi_flt_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"cbbcbl",
 					'(' /* ) */ ,
 					&at->bam_sel_expr,
 					&ordering,
@@ -1206,7 +1278,9 @@ equality_match:;
 		break;
 
 	case LDAP_FILTER_PRESENT:
-		backsql_strfcat( &bsi->bsi_flt_where, "lbl",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"lbl",
 				(ber_len_t)STRLENOF( "NOT (" /* ) */),
 					"NOT (", /* ) */
 				&at->bam_sel_expr, 
@@ -1233,7 +1307,9 @@ equality_match:;
 	default:
 		/* unhandled filter type; should not happen */
 		assert( 0 );
-		backsql_strfcat( &bsi->bsi_flt_where, "l",
+		backsql_strfcat_x( &bsi->bsi_flt_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"l",
 				(ber_len_t)STRLENOF( "8=8" ), "8=8" );
 		break;
 
@@ -1248,10 +1324,10 @@ equality_match:;
 static int
 backsql_srch_query( backsql_srch_info *bsi, struct berval *query )
 {
-	backsql_info	*bi = (backsql_info *)bsi->bsi_op->o_bd->be_private;
-	int		rc;
+	backsql_info		*bi = (backsql_info *)bsi->bsi_op->o_bd->be_private;
+	int			rc;
 
-	assert( query );
+	assert( query != NULL );
 	BER_BVZERO( query );
 
 	bsi->bsi_use_subtree_shortcut = 0;
@@ -1267,7 +1343,9 @@ backsql_srch_query( backsql_srch_info *bsi, struct berval *query )
 	BER_BVZERO( &bsi->bsi_flt_where.bb_val );
 	bsi->bsi_flt_where.bb_len = 0;
 
-	backsql_strfcat( &bsi->bsi_sel, "lbcbc",
+	backsql_strfcat_x( &bsi->bsi_sel,
+			bsi->bsi_op->o_tmpmemctx,
+			"lbcbc",
 			(ber_len_t)STRLENOF( "SELECT DISTINCT ldap_entries.id," ),
 				"SELECT DISTINCT ldap_entries.id,", 
 			&bsi->bsi_oc->bom_keytbl, 
@@ -1276,7 +1354,9 @@ backsql_srch_query( backsql_srch_info *bsi, struct berval *query )
 			',' );
 
 	if ( !BER_BVISNULL( &bi->sql_strcast_func ) ) {
-		backsql_strfcat( &bsi->bsi_sel, "blbl",
+		backsql_strfcat_x( &bsi->bsi_sel,
+				bsi->bsi_op->o_tmpmemctx,
+				"blbl",
 				&bi->sql_strcast_func, 
 				(ber_len_t)STRLENOF( "('" /* ') */ ),
 					"('" /* ') */ ,
@@ -1284,25 +1364,28 @@ backsql_srch_query( backsql_srch_info *bsi, struct berval *query )
 				(ber_len_t)STRLENOF( /* (' */ "')" ),
 					/* (' */ "')" );
 	} else {
-		backsql_strfcat( &bsi->bsi_sel, "cbc",
+		backsql_strfcat_x( &bsi->bsi_sel,
+				bsi->bsi_op->o_tmpmemctx,
+				"cbc",
 				'\'',
 				&bsi->bsi_oc->bom_oc->soc_cname,
 				'\'' );
 	}
-	backsql_strfcat( &bsi->bsi_sel, "l",
-			(ber_len_t)STRLENOF( " " BACKSQL_ALIASING 
-					BACKSQL_ALIASING_QUOTE "objectClass" BACKSQL_ALIASING_QUOTE 
-					",ldap_entries.dn " BACKSQL_ALIASING BACKSQL_ALIASING_QUOTE "dn" BACKSQL_ALIASING_QUOTE ),
-				" " BACKSQL_ALIASING 
-					BACKSQL_ALIASING_QUOTE "objectClass" BACKSQL_ALIASING_QUOTE
-					",ldap_entries.dn " BACKSQL_ALIASING BACKSQL_ALIASING_QUOTE "dn" BACKSQL_ALIASING_QUOTE );
 
-	backsql_strfcat( &bsi->bsi_from, "lb",
+	backsql_strfcat_x( &bsi->bsi_sel,
+			bsi->bsi_op->o_tmpmemctx,
+			"b",
+			&bi->sql_dn_oc_aliasing );
+	backsql_strfcat_x( &bsi->bsi_from,
+			bsi->bsi_op->o_tmpmemctx,
+			"lb",
 			(ber_len_t)STRLENOF( " FROM ldap_entries," ),
 				" FROM ldap_entries,",
 			&bsi->bsi_oc->bom_keytbl );
 
-	backsql_strfcat( &bsi->bsi_join_where, "lbcbl",
+	backsql_strfcat_x( &bsi->bsi_join_where,
+			bsi->bsi_op->o_tmpmemctx,
+			"lbcbl",
 			(ber_len_t)STRLENOF( " WHERE " ), " WHERE ",
 			&bsi->bsi_oc->bom_keytbl,
 			'.',
@@ -1313,12 +1396,16 @@ backsql_srch_query( backsql_srch_info *bsi, struct berval *query )
 	switch ( bsi->bsi_scope ) {
 	case LDAP_SCOPE_BASE:
 		if ( BACKSQL_CANUPPERCASE( bi ) ) {
-			backsql_strfcat( &bsi->bsi_join_where, "bl",
+			backsql_strfcat_x( &bsi->bsi_join_where,
+					bsi->bsi_op->o_tmpmemctx, 
+					"bl",
 					&bi->sql_upper_func,
 					(ber_len_t)STRLENOF( "(ldap_entries.dn)=?" ),
 						"(ldap_entries.dn)=?" );
 		} else {
-			backsql_strfcat( &bsi->bsi_join_where, "l",
+			backsql_strfcat_x( &bsi->bsi_join_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"l",
 					(ber_len_t)STRLENOF( "ldap_entries.dn=?" ),
 						"ldap_entries.dn=?" );
 		}
@@ -1326,32 +1413,36 @@ backsql_srch_query( backsql_srch_info *bsi, struct berval *query )
 		
 	case BACKSQL_SCOPE_BASE_LIKE:
 		if ( BACKSQL_CANUPPERCASE( bi ) ) {
-			backsql_strfcat( &bsi->bsi_join_where, "bl",
+			backsql_strfcat_x( &bsi->bsi_join_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"bl",
 					&bi->sql_upper_func,
 					(ber_len_t)STRLENOF( "(ldap_entries.dn) LIKE ?" ),
 						"(ldap_entries.dn) LIKE ?" );
 		} else {
-			backsql_strfcat( &bsi->bsi_join_where, "l",
+			backsql_strfcat_x( &bsi->bsi_join_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"l",
 					(ber_len_t)STRLENOF( "ldap_entries.dn LIKE ?" ),
 						"ldap_entries.dn LIKE ?" );
 		}
 		break;
 		
 	case LDAP_SCOPE_ONELEVEL:
-		backsql_strfcat( &bsi->bsi_join_where, "l",
+		backsql_strfcat_x( &bsi->bsi_join_where,
+				bsi->bsi_op->o_tmpmemctx,
+				"l",
 				(ber_len_t)STRLENOF( "ldap_entries.parent=?" ),
 					"ldap_entries.parent=?" );
 		break;
 
-#ifdef LDAP_SCOPE_SUBORDINATE
 	case LDAP_SCOPE_SUBORDINATE:
-#endif /* LDAP_SCOPE_SUBORDINATE */
 	case LDAP_SCOPE_SUBTREE:
 		if ( BACKSQL_USE_SUBTREE_SHORTCUT( bi ) ) {
 			int		i;
 			BackendDB	*bd = bsi->bsi_op->o_bd;
 
-			assert( bd->be_nsuffix );
+			assert( bd->be_nsuffix != NULL );
 
 			for ( i = 0; !BER_BVISNULL( &bd->be_nsuffix[ i ] ); i++ )
 			{
@@ -1369,20 +1460,29 @@ backsql_srch_query( backsql_srch_info *bsi, struct berval *query )
 
 		if ( bsi->bsi_use_subtree_shortcut ) {
 			/* Skip the base DN filter, as every entry will match it */
-			backsql_strfcat( &bsi->bsi_join_where, "l",
+			backsql_strfcat_x( &bsi->bsi_join_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"l",
 					(ber_len_t)STRLENOF( "9=9"), "9=9");
 
 		} else if ( !BER_BVISNULL( &bi->sql_subtree_cond ) ) {
-			backsql_strfcat( &bsi->bsi_join_where, "b", &bi->sql_subtree_cond );
+			backsql_strfcat_x( &bsi->bsi_join_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"b",
+					&bi->sql_subtree_cond );
 
 		} else if ( BACKSQL_CANUPPERCASE( bi ) ) {
-			backsql_strfcat( &bsi->bsi_join_where, "bl",
+			backsql_strfcat_x( &bsi->bsi_join_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"bl",
 					&bi->sql_upper_func,
 					(ber_len_t)STRLENOF( "(ldap_entries.dn) LIKE ?" ),
 						"(ldap_entries.dn) LIKE ?"  );
 
 		} else {
-			backsql_strfcat( &bsi->bsi_join_where, "l",
+			backsql_strfcat_x( &bsi->bsi_join_where,
+					bsi->bsi_op->o_tmpmemctx,
+					"l",
 					(ber_len_t)STRLENOF( "ldap_entries.dn LIKE ?" ),
 						"ldap_entries.dn LIKE ?" );
 		}
@@ -1397,7 +1497,9 @@ backsql_srch_query( backsql_srch_info *bsi, struct berval *query )
 	if ( rc > 0 ) {
 		struct berbuf	bb = BB_NULL;
 
-		backsql_strfcat( &bb, "bbblb",
+		backsql_strfcat_x( &bb,
+				bsi->bsi_op->o_tmpmemctx,
+				"bbblb",
 				&bsi->bsi_sel.bb_val,
 				&bsi->bsi_from.bb_val, 
 				&bsi->bsi_join_where.bb_val,
@@ -1415,16 +1517,16 @@ backsql_srch_query( backsql_srch_info *bsi, struct berval *query )
 		BER_BVZERO( query );
 	}
  
-	free( bsi->bsi_sel.bb_val.bv_val );
+	bsi->bsi_op->o_tmpfree( bsi->bsi_sel.bb_val.bv_val, bsi->bsi_op->o_tmpmemctx );
 	BER_BVZERO( &bsi->bsi_sel.bb_val );
 	bsi->bsi_sel.bb_len = 0;
-	free( bsi->bsi_from.bb_val.bv_val );
+	bsi->bsi_op->o_tmpfree( bsi->bsi_from.bb_val.bv_val, bsi->bsi_op->o_tmpmemctx );
 	BER_BVZERO( &bsi->bsi_from.bb_val );
 	bsi->bsi_from.bb_len = 0;
-	free( bsi->bsi_join_where.bb_val.bv_val );
+	bsi->bsi_op->o_tmpfree( bsi->bsi_join_where.bb_val.bv_val, bsi->bsi_op->o_tmpmemctx );
 	BER_BVZERO( &bsi->bsi_join_where.bb_val );
 	bsi->bsi_join_where.bb_len = 0;
-	free( bsi->bsi_flt_where.bb_val.bv_val );
+	bsi->bsi_op->o_tmpfree( bsi->bsi_flt_where.bb_val.bv_val, bsi->bsi_op->o_tmpmemctx );
 	BER_BVZERO( &bsi->bsi_flt_where.bb_val );
 	bsi->bsi_flt_where.bb_len = 0;
 	
@@ -1464,6 +1566,12 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 	Debug( LDAP_DEBUG_TRACE, "==>backsql_oc_get_candidates(): oc=\"%s\"\n",
 			BACKSQL_OC_NAME( oc ), 0, 0 );
 
+	/* check for abandon */
+	if ( op->o_abandon ) {
+		bsi->bsi_status = SLAPD_ABANDON;
+		return BACKSQL_AVL_STOP;
+	}
+
 	if ( bsi->bsi_n_candidates == -1 ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_oc_get_candidates(): "
 			"unchecked limit has been overcome\n", 0, 0, 0 );
@@ -1512,7 +1620,7 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 			query.bv_val, 0, 0 );
 
 	rc = backsql_Prepare( bsi->bsi_dbh, &sth, query.bv_val, 0 );
-	free( query.bv_val );
+	bsi->bsi_op->o_tmpfree( query.bv_val, bsi->bsi_op->o_tmpmemctx );
 	BER_BVZERO( &query );
 	if ( rc != SQL_SUCCESS ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_oc_get_candidates(): "
@@ -1569,9 +1677,7 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 		}
 		break;
 
-#ifdef LDAP_SCOPE_SUBORDINATE
 	case LDAP_SCOPE_SUBORDINATE:
-#endif /* LDAP_SCOPE_SUBORDINATE */
 	case LDAP_SCOPE_SUBTREE:
 	{
 		/* if short-cutting the search base,
@@ -1610,11 +1716,9 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 				tmp_base_ndn[ i ] = bsi->bsi_base_ndn->bv_val[ j ];
 			}
 
-#ifdef LDAP_SCOPE_SUBORDINATE
 			if ( bsi->bsi_scope == LDAP_SCOPE_SUBORDINATE ) {
 				tmp_base_ndn[ i++ ] = ',';
 			}
-#endif /* LDAP_SCOPE_SUBORDINATE */
 
 			tmp_base_ndn[ i ] = '%';
 			tmp_base_ndn[ i + 1 ] = '\0';
@@ -1624,11 +1728,9 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 
 			tmp_base_ndn[ i++ ] = '%';
 
-#ifdef LDAP_SCOPE_SUBORDINATE
 			if ( bsi->bsi_scope == LDAP_SCOPE_SUBORDINATE ) {
 				tmp_base_ndn[ i++ ] = ',';
 			}
-#endif /* LDAP_SCOPE_SUBORDINATE */
 
 			AC_MEMCPY( &tmp_base_ndn[ i ], bsi->bsi_base_ndn->bv_val,
 				bsi->bsi_base_ndn->bv_len + 1 );
@@ -1640,13 +1742,10 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 			ldap_pvt_str2upper( tmp_base_ndn );
 		}
 
-#ifdef LDAP_SCOPE_SUBORDINATE
 		if ( bsi->bsi_scope == LDAP_SCOPE_SUBORDINATE ) {
 			Debug( LDAP_DEBUG_TRACE, "(children)dn: \"%s\"\n",
 				tmp_base_ndn, 0, 0 );
-		} else 
-#endif /* LDAP_SCOPE_SUBORDINATE */
-		{
+		} else {
 			Debug( LDAP_DEBUG_TRACE, "(sub)dn: \"%s\"\n",
 				tmp_base_ndn, 0, 0 );
 		}
@@ -1696,7 +1795,7 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 		return BACKSQL_AVL_CONTINUE;
 	}
 
-	backsql_BindRowAsStrings( sth, &row );
+	backsql_BindRowAsStrings_x( sth, &row, bsi->bsi_op->o_tmpmemctx );
 	rc = SQLFetch( sth );
 	for ( ; BACKSQL_SUCCESS( rc ); rc = SQLFetch( sth ) ) {
 		struct berval		dn, pdn, ndn;
@@ -1719,21 +1818,23 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 		}
 
 		if ( bi->sql_baseObject && dn_match( &ndn, &bi->sql_baseObject->e_nname ) ) {
-			op->o_tmpfree( pdn.bv_val, op->o_tmpmemctx );
-			op->o_tmpfree( ndn.bv_val, op->o_tmpmemctx );
-			continue;
+			goto cleanup;
 		}
 
-		c_id = (backsql_entryID *)ch_calloc( 1, 
-				sizeof( backsql_entryID ) );
+		c_id = (backsql_entryID *)op->o_tmpcalloc( 1, 
+				sizeof( backsql_entryID ), op->o_tmpmemctx );
 #ifdef BACKSQL_ARBITRARY_KEY
 		ber_str2bv_x( row.cols[ 0 ], 0, 1, &c_id->eid_id,
 				op->o_tmpmemctx );
 		ber_str2bv_x( row.cols[ 1 ], 0, 1, &c_id->eid_keyval,
 				op->o_tmpmemctx );
 #else /* ! BACKSQL_ARBITRARY_KEY */
-		c_id->eid_id = strtol( row.cols[ 0 ], NULL, 0 );
-		c_id->eid_keyval = strtol( row.cols[ 1 ], NULL, 0 );
+		if ( lutil_atoulx( &c_id->eid_id, row.cols[ 0 ], 0 ) != 0 ) {
+			goto cleanup;
+		}
+		if ( lutil_atoulx( &c_id->eid_keyval, row.cols[ 1 ], 0 ) != 0 ) {
+			goto cleanup;
+		}
 #endif /* ! BACKSQL_ARBITRARY_KEY */
 		c_id->eid_oc_id = bsi->bsi_oc->bom_id;
 
@@ -1761,8 +1862,20 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 		if ( bsi->bsi_n_candidates == -1 ) {
 			break;
 		}
+		continue;
+
+cleanup:;
+		if ( !BER_BVISNULL( &pdn ) ) {
+			op->o_tmpfree( pdn.bv_val, op->o_tmpmemctx );
+		}
+		if ( !BER_BVISNULL( &ndn ) ) {
+			op->o_tmpfree( ndn.bv_val, op->o_tmpmemctx );
+		}
+		if ( c_id != NULL ) {
+			ch_free( c_id );
+		}
 	}
-	backsql_FreeRow( &row );
+	backsql_FreeRow_x( &row, bsi->bsi_op->o_tmpmemctx );
 	SQLFreeStmt( sth, SQL_DROP );
 
 	Debug( LDAP_DEBUG_TRACE, "<==backsql_oc_get_candidates(): %d\n",
@@ -1788,7 +1901,7 @@ backsql_search( Operation *op, SlapReply *rs )
 	Debug( LDAP_DEBUG_TRACE, "==>backsql_search(): "
 		"base=\"%s\", filter=\"%s\", scope=%d,", 
 		op->o_req_ndn.bv_val,
-		op->ors_filterstr.bv_val ? op->ors_filterstr.bv_val : "(no filter)",
+		op->ors_filterstr.bv_val,
 		op->ors_scope );
 	Debug( LDAP_DEBUG_TRACE, " deref=%d, attrsonly=%d, "
 		"attributes to load: %s\n",
@@ -1828,7 +1941,6 @@ backsql_search( Operation *op, SlapReply *rs )
 	bsi.bsi_e = &base_entry;
 	rs->sr_err = backsql_init_search( &bsi, &op->o_req_ndn,
 			op->ors_scope,
-			op->ors_slimit, op->ors_tlimit,
 			stoptime, op->ors_filter,
 			dbh, op, rs, op->ors_attrs,
 			( BACKSQL_ISF_MATCHED | BACKSQL_ISF_GET_ENTRY ) );
@@ -1849,12 +1961,15 @@ backsql_search( Operation *op, SlapReply *rs )
 			}
 			break;
 		}
+
+		/* an entry was created; free it */
+		entry_clean( bsi.bsi_e );
+
 		/* fall thru */
 
 	default:
-#ifdef SLAP_ACL_HONOR_DISCLOSE
 		if ( !BER_BVISNULL( &base_entry.e_nname )
-				&& ! access_allowed( op, &base_entry,
+				&& !access_allowed( op, &base_entry,
 					slap_schema.si_ad_entry, NULL,
 					ACL_DISCLOSE, NULL ) )
 		{
@@ -1866,13 +1981,20 @@ backsql_search( Operation *op, SlapReply *rs )
 			rs->sr_matched = NULL;
 			rs->sr_text = NULL;
 		}
-#endif /* SLAP_ACL_HONOR_DISCLOSE */
 
 		send_ldap_result( op, rs );
-		goto done;
 
+		if ( rs->sr_ref ) {
+			ber_bvarray_free( rs->sr_ref );
+			rs->sr_ref = NULL;
+		}
+
+		if ( !BER_BVISNULL( &base_entry.e_nname ) ) {
+			entry_clean( &base_entry );
+		}
+
+		goto done;
 	}
-#ifdef SLAP_ACL_HONOR_DISCLOSE
 	/* NOTE: __NEW__ "search" access is required
 	 * on searchBase object */
 	{
@@ -1903,7 +2025,6 @@ backsql_search( Operation *op, SlapReply *rs )
 			goto done;
 		}
 	}
-#endif /* SLAP_ACL_HONOR_DISCLOSE */
 
 	bsi.bsi_e = NULL;
 
@@ -1943,6 +2064,13 @@ backsql_search( Operation *op, SlapReply *rs )
 		 */
 		avl_apply( bi->sql_oc_by_oc, backsql_oc_get_candidates,
 				&bsi, BACKSQL_AVL_STOP, AVL_INORDER );
+
+		/* check for abandon */
+		if ( op->o_abandon ) {
+			eid = bsi.bsi_id_list;
+			rs->sr_err = SLAPD_ABANDON;
+			goto send_results;
+		}
 	}
 
 	if ( op->ors_limit != NULL	/* isroot == FALSE */
@@ -1973,7 +2101,8 @@ backsql_search( Operation *op, SlapReply *rs )
 
 		/* check for abandon */
 		if ( op->o_abandon ) {
-			break;
+			rs->sr_err = SLAPD_ABANDON;
+			goto send_results;
 		}
 
 		/* check time limit */
@@ -1983,10 +2112,7 @@ backsql_search( Operation *op, SlapReply *rs )
 			rs->sr_err = LDAP_TIMELIMIT_EXCEEDED;
 			rs->sr_ctrls = NULL;
 			rs->sr_ref = rs->sr_v2ref;
-			rs->sr_err = (rs->sr_v2ref == NULL) ? LDAP_SUCCESS
-				: LDAP_REFERRAL;
-			send_ldap_result( op, rs );
-			goto end_of_search;
+			goto send_results;
 		}
 
 #ifdef BACKSQL_ARBITRARY_KEY
@@ -2020,15 +2146,12 @@ backsql_search( Operation *op, SlapReply *rs )
 			/* fall thru */
 		}
 
-#ifdef LDAP_SCOPE_SUBORDINATE
 		case LDAP_SCOPE_SUBORDINATE:
 			/* discard the baseObject entry */
 			if ( dn_match( &eid->eid_ndn, &op->o_req_ndn ) ) {
 				goto next_entry2;
 			}
-		/* FALLTHRU */
-#endif /* LDAP_SCOPE_SUBORDINATE */
-
+			/* FALLTHRU */
 		case LDAP_SCOPE_SUBTREE:
 			/* FIXME: this should never fail... */
 			if ( !dnIsSuffix( &eid->eid_ndn, &op->o_req_ndn ) ) {
@@ -2075,7 +2198,6 @@ backsql_search( Operation *op, SlapReply *rs )
 				rc = backsql_init_search( &bsi2,
 						&e->e_nname,
 						LDAP_SCOPE_BASE, 
-						SLAP_NO_LIMIT, SLAP_NO_LIMIT,
 						(time_t)(-1), NULL,
 						dbh, op, rs, NULL,
 						BACKSQL_ISF_GET_ENTRY );
@@ -2087,7 +2209,7 @@ backsql_search( Operation *op, SlapReply *rs )
 					} else {
 						rs->sr_err = LDAP_OTHER;
 					}
-					entry_clean( &user_entry2 );
+					backsql_entry_clean( op, &user_entry2 );
 				}
 				if ( bsi2.bsi_attrs != NULL ) {
 					op->o_tmpfree( bsi2.bsi_attrs,
@@ -2128,7 +2250,7 @@ backsql_search( Operation *op, SlapReply *rs )
 		 * filter_has_subordinates()
 		 */
 		if ( bsi.bsi_flags & BSQL_SF_FILTER_HASSUBORDINATE ) {
-			rc = backsql_has_children( bi, dbh, &e->e_nname );
+			rc = backsql_has_children( op, dbh, &e->e_nname );
 
 			switch ( rc ) {
 			case LDAP_COMPARE_TRUE:
@@ -2187,16 +2309,15 @@ backsql_search( Operation *op, SlapReply *rs )
 			rs->sr_attrs = op->ors_attrs;
 			rs->sr_operational_attrs = NULL;
 			rs->sr_entry = e;
-			if ( e == &user_entry ) {
-				rs->sr_flags = REP_ENTRY_MODIFIABLE;
-			}
+			rs->sr_flags = ( e == &user_entry ) ? REP_ENTRY_MODIFIABLE : 0;
 			/* FIXME: need the whole entry (ITS#3480) */
-			sres = send_search_entry( op, rs );
+			rs->sr_err = send_search_entry( op, rs );
 			rs->sr_entry = NULL;
 			rs->sr_attrs = NULL;
 			rs->sr_operational_attrs = NULL;
 
-			if ( sres == -1 ) {
+			switch ( rs->sr_err ) {
+			case LDAP_UNAVAILABLE:
 				/*
 				 * FIXME: send_search_entry failed;
 				 * better stop
@@ -2204,30 +2325,21 @@ backsql_search( Operation *op, SlapReply *rs )
 				Debug( LDAP_DEBUG_TRACE, "backsql_search(): "
 					"connection lost\n", 0, 0, 0 );
 				goto end_of_search;
+
+			case LDAP_SIZELIMIT_EXCEEDED:
+				goto send_results;
 			}
 		}
 
 next_entry:;
 		if ( e == &user_entry ) {
-			entry_clean( &user_entry );
+			backsql_entry_clean( op, &user_entry );
 		}
 
 next_entry2:;
-		if ( op->ors_slimit != SLAP_NO_LIMIT
-				&& rs->sr_nentries >= op->ors_slimit )
-		{
-			rs->sr_err = LDAP_SIZELIMIT_EXCEEDED;
-			send_ldap_result( op, rs );
-			goto end_of_search;
-		}
 	}
 
 end_of_search:;
-	entry_clean( &base_entry );
-
-	/* in case we got here accidentally */
-	entry_clean( &user_entry );
-
 	if ( rs->sr_nentries > 0 ) {
 		rs->sr_ref = rs->sr_v2ref;
 		rs->sr_err = (rs->sr_v2ref == NULL) ? LDAP_SUCCESS
@@ -2236,7 +2348,22 @@ end_of_search:;
 	} else {
 		rs->sr_err = bsi.bsi_status;
 	}
-	send_ldap_result( op, rs );
+
+send_results:;
+	if ( rs->sr_err != SLAPD_ABANDON ) {
+		send_ldap_result( op, rs );
+	}
+
+	/* cleanup in case of abandon */
+	for ( ; eid != NULL; 
+			eid = backsql_free_entryID( op,
+				eid, eid == &bsi.bsi_base_id ? 0 : 1 ) )
+		;
+
+	backsql_entry_clean( op, &base_entry );
+
+	/* in case we got here accidentally */
+	backsql_entry_clean( op, &user_entry );
 
 	if ( rs->sr_v2ref ) {
 		ber_bvarray_free( rs->sr_v2ref );
@@ -2284,7 +2411,8 @@ done:;
 	}
 
 	Debug( LDAP_DEBUG_TRACE, "<==backsql_search()\n", 0, 0, 0 );
-	return 0;
+
+	return rs->sr_err;
 }
 
 /* return LDAP_SUCCESS IFF we can retrieve the specified entry.
@@ -2321,7 +2449,6 @@ backsql_entry_get(
 	rc = backsql_init_search( &bsi,
 			ndn,
 			LDAP_SCOPE_BASE, 
-			SLAP_NO_LIMIT, SLAP_NO_LIMIT,
 			(time_t)(-1), NULL,
 			dbh, op, &rs, at ? anlist : NULL,
 			BACKSQL_ISF_GET_ENTRY );
@@ -2377,3 +2504,39 @@ return_results:;
 	return rc;
 }
 
+void
+backsql_entry_clean(
+		Operation	*op,
+		Entry		*e )
+{
+	void *ctx;
+
+	ctx = ldap_pvt_thread_pool_context();
+
+	if ( ctx == NULL || ctx != op->o_tmpmemctx ) {
+		if ( !BER_BVISNULL( &e->e_name ) ) {
+			op->o_tmpfree( e->e_name.bv_val, op->o_tmpmemctx );
+			BER_BVZERO( &e->e_name );
+		}
+
+		if ( !BER_BVISNULL( &e->e_nname ) ) {
+			op->o_tmpfree( e->e_nname.bv_val, op->o_tmpmemctx );
+			BER_BVZERO( &e->e_nname );
+		}
+	}
+
+	entry_clean( e );
+}
+
+int
+backsql_entry_release(
+		Operation	*op,
+		Entry		*e,
+		int		rw )
+{
+	backsql_entry_clean( op, e );
+
+	ch_free( e );
+
+	return 0;
+}
