@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/daemon.c,v 1.380 2006/04/06 02:34:37 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/daemon.c,v 1.380.2.1 2006/05/15 17:04:41 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2006 The OpenLDAP Foundation.
@@ -339,9 +339,13 @@ static struct slap_daemon {
 static char** slapd_srvurls = NULL;
 static SLPHandle slapd_hslp = 0;
 int slapd_register_slp = 0;
+char *slapd_slp_attrs = NULL;
+
+static SLPError slapd_slp_cookie;
 
 void slapd_slp_init( const char* urls ) {
 	int i;
+	SLPError err;
 
 	slapd_srvurls = ldap_str2charray( urls, " " );
 
@@ -376,7 +380,12 @@ void slapd_slp_init( const char* urls ) {
 	}
 
 	/* open the SLP handle */
-	SLPOpen( "en", 0, &slapd_hslp );
+	err = SLPOpen( "en", 0, &slapd_hslp );
+
+	if (err != SLP_OK) {
+		Debug( LDAP_DEBUG_CONNS, "daemon: SLPOpen() failed with %ld\n",
+			(long)err, 0, 0 );
+	}
 }
 
 void slapd_slp_deinit() {
@@ -394,11 +403,13 @@ void slapd_slp_regreport(
 	SLPError errcode,
 	void* cookie )
 {
-	/* empty report */
+	/* return the error code in the cookie */
+	*(SLPError*)cookie = errcode; 
 }
 
 void slapd_slp_reg() {
 	int i;
+	SLPError err;
 
 	if( slapd_srvurls == NULL ) return;
 
@@ -408,28 +419,41 @@ void slapd_slp_reg() {
 		    strncmp( slapd_srvurls[i], LDAPS_SRVTYPE_PREFIX,
 				sizeof( LDAPS_SRVTYPE_PREFIX ) - 1 ) == 0 )
 		{
-			SLPReg( slapd_hslp,
+			err = SLPReg( slapd_hslp,
 				slapd_srvurls[i],
 				SLP_LIFETIME_MAXIMUM,
 				"ldap",
-				"",
-				1,
+					(slapd_slp_attrs) ? slapd_slp_attrs : "",
+					SLP_TRUE,
 				slapd_slp_regreport,
-				NULL );
+					&slapd_slp_cookie );
+
+			if (err != SLP_OK || slapd_slp_cookie != SLP_OK) {
+				Debug( LDAP_DEBUG_CONNS,
+					"daemon: SLPReg(%s) failed with %ld, cookie = %ld\n",
+					slapd_srvurls[i], (long)err, (long)slapd_slp_cookie );
+			}	
 		}
 	}
 }
 
 void slapd_slp_dereg() {
 	int i;
+	SLPError err;
 
 	if( slapd_srvurls == NULL ) return;
 
 	for( i=0; slapd_srvurls[i] != NULL; i++ ) {
-		SLPDereg( slapd_hslp,
+		err = SLPDereg( slapd_hslp,
 			slapd_srvurls[i],
 			slapd_slp_regreport,
-			NULL );
+				&slapd_slp_cookie );
+		
+		if (err != SLP_OK || slapd_slp_cookie != SLP_OK) {
+			Debug( LDAP_DEBUG_CONNS,
+				"daemon: SLPDereg(%s) failed with %ld, cookie = %ld\n",
+				slapd_srvurls[i], (long)err, (long)slapd_slp_cookie );
+		}
 	}
 }
 #endif /* HAVE_SLP */
@@ -807,7 +831,8 @@ static int slap_get_listener_addresses(
 			sap[i]->sa_family = AF_INET;
 			((struct sockaddr_in *)sap[i])->sin_port = htons(port);
 			AC_MEMCPY( &((struct sockaddr_in *)sap[i])->sin_addr,
-				he ? he->h_addr_list[i] : &in, sizeof(struct in_addr) );
+				he ? (struct in_addr *)he->h_addr_list[i] : &in,
+				sizeof(struct in_addr) );
 		}
 		sap[i] = NULL;
 #endif
@@ -1705,7 +1730,7 @@ slapd_daemon_task(
 		struct timeval		tv;
 		struct timeval		*tvp;
 
-		struct timeval		*cat;
+		struct timeval		cat;
 		time_t				tdelta = 1;
 		struct re_s*		rtask;
 		now = slap_get_time();
@@ -1786,7 +1811,7 @@ slapd_daemon_task(
 
 		ldap_pvt_thread_mutex_lock( &slapd_rq.rq_mutex );
 		rtask = ldap_pvt_runqueue_next_sched( &slapd_rq, &cat );
-		while ( cat && cat->tv_sec && cat->tv_sec <= now ) {
+		while ( rtask && cat.tv_sec && cat.tv_sec <= now ) {
 			if ( ldap_pvt_runqueue_isrunning( &slapd_rq, rtask )) {
 				ldap_pvt_runqueue_resched( &slapd_rq, rtask, 0 );
 			} else {
@@ -1794,15 +1819,15 @@ slapd_daemon_task(
 				ldap_pvt_runqueue_resched( &slapd_rq, rtask, 0 );
 				ldap_pvt_thread_mutex_unlock( &slapd_rq.rq_mutex );
 				ldap_pvt_thread_pool_submit( &connection_pool,
-											rtask->routine, (void *) rtask );
+					rtask->routine, (void *) rtask );
 				ldap_pvt_thread_mutex_lock( &slapd_rq.rq_mutex );
 			}
 			rtask = ldap_pvt_runqueue_next_sched( &slapd_rq, &cat );
 		}
 		ldap_pvt_thread_mutex_unlock( &slapd_rq.rq_mutex );
 
-		if ( cat && cat->tv_sec ) {
-			time_t diff = difftime( cat->tv_sec, now );
+		if ( rtask && cat.tv_sec ) {
+			time_t diff = difftime( cat.tv_sec, now );
 			if ( diff == 0 ) diff = tdelta;
 			if ( tvp == NULL || diff < tv.tv_sec ) {
 				tv.tv_sec = diff;
@@ -2069,7 +2094,7 @@ slapd_daemon_task(
 #endif
 
 		for (i=0; i<ns; i++) {
-			int rc = 1, fd, waswrite = 0;
+			int rc = 1, fd;
 
 			if ( SLAP_EVENT_IS_LISTENER(i) ) {
 #ifdef SLAP_LIGHTWEIGHT_DISPATCHER
@@ -2099,7 +2124,7 @@ slapd_daemon_task(
 						"daemon: write active on %d\n",
 						fd, 0, 0 );
 
-					waswrite = 1;
+					SLAP_EVENT_CLR_WRITE( i );
 
 					/*
 					 * NOTE: it is possible that the connection was closed
@@ -2111,12 +2136,13 @@ slapd_daemon_task(
 						continue;
 					}
 				}
-				/* If event is a read or an error */
-				if( SLAP_EVENT_IS_READ( i ) || !waswrite ) {
+				/* If event is a read */
+				if( SLAP_EVENT_IS_READ( i )) {
 					Debug( LDAP_DEBUG_CONNS,
 						"daemon: read active on %d\n",
 						fd, 0, 0 );
 
+					SLAP_EVENT_CLR_READ( i );
 #ifdef SLAP_LIGHTWEIGHT_DISPATCHER
 					connection_read_activate( fd );
 #else
@@ -2128,6 +2154,9 @@ slapd_daemon_task(
 					 */
 					connection_read( fd );
 #endif
+				} else {
+					Debug( LDAP_DEBUG_CONNS,
+						"daemon: hangup on %d\n", fd, 0, 0 );
 				}
 			}
 		}
