@@ -1,8 +1,8 @@
 /* rwmdn.c - massages dns */
-/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/rwmdn.c,v 1.11.2.2 2005/01/24 21:28:53 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/rwmdn.c,v 1.18 2006/01/03 22:12:25 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2005 The OpenLDAP Foundation.
+ * Copyright 1999-2006 The OpenLDAP Foundation.
  * Portions Copyright 1999-2003 Howard Chu.
  * Portions Copyright 2000-2003 Pierangelo Masarati.
  * All rights reserved.
@@ -157,13 +157,14 @@ rwm_dn_massage(
 {
 	int		rc = 0;
 	struct berval	mdn;
+	static char	*dmy = "";
 
-	assert( dc );
-	assert( in );
-	assert( dn );
+	assert( dc != NULL );
+	assert( in != NULL );
+	assert( dn != NULL );
 
 	rc = rewrite_session( dc->rwmap->rwm_rw, dc->ctx,
-			( in->bv_len ? in->bv_val : "" ), 
+			( in->bv_val ? in->bv_val : dmy ), 
 			dc->conn, &mdn.bv_val );
 	switch ( rc ) {
 	case REWRITE_REGEXEC_OK:
@@ -197,6 +198,14 @@ rwm_dn_massage(
 		break;
 	}
 
+	if ( mdn.bv_val == dmy ) {
+		BER_BVZERO( &mdn );
+	}
+
+	if ( dn->bv_val == dmy ) {
+		BER_BVZERO( dn );
+	}
+
 	return rc;
 }
 
@@ -216,9 +225,9 @@ rwm_dn_massage(
 	int     	i, src, dst;
 	struct berval	tmpin;
 
-	assert( dc );
-	assert( in );
-	assert( dn );
+	assert( dc != NULL );
+	assert( in != NULL );
+	assert( dn != NULL );
 
 	BER_BVZERO( dn );
 
