@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/slapadd.c,v 1.11.2.3 2005/02/02 17:32:44 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/slapadd.c,v 1.36 2006/01/08 19:18:14 lukeh Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2005 The OpenLDAP Foundation.
+ * Copyright 1998-2006 The OpenLDAP Foundation.
  * Portions Copyright 1998-2003 Kurt D. Zeilenga.
  * Portions Copyright 2003 IBM Corporation.
  * All rights reserved.
@@ -40,96 +40,45 @@
 
 static char csnbuf[ LDAP_LUTIL_CSNSTR_BUFSIZE ];
 static char maxcsnbuf[ LDAP_LUTIL_CSNSTR_BUFSIZE ];
-static const char *progname = "slapadd";
-
-static ldap_pvt_thread_cond_t put_cond1;
-static ldap_pvt_thread_cond_t put_cond2;
-static ldap_pvt_thread_mutex_t put_mutex1;
-static ldap_pvt_thread_mutex_t put_mutex2;
-static Entry *put_e;
-static struct berval bvtext;
-static int put_lineno;
-static int put_rc;
-
-static int use_thread = 0;	/*FIXME need a new switch for this */
-
-static void *do_put(void *ptr)
-{
-	ID id;
-	Entry *e;
-	int lineno;
-
-	ldap_pvt_thread_mutex_lock( &put_mutex1 );
-	do {
-		ldap_pvt_thread_cond_wait( &put_cond1, &put_mutex1 );
-		if ( put_rc ) {
-			break;
-		}
-		ldap_pvt_thread_mutex_lock( &put_mutex2 );
-		ldap_pvt_thread_cond_signal( &put_cond2 );
-		ldap_pvt_thread_mutex_unlock( &put_mutex2 );
-
-		e = put_e;
-		lineno = put_lineno;
-
-		if ( !dryrun ) {
-			id = be->be_entry_put( be, e, &bvtext );
-			if( id == NOID ) {
-				fprintf( stderr, "%s: could not add entry dn=\"%s\" "
-								 "(line=%d): %s\n", progname, e->e_dn,
-								 lineno, bvtext.bv_val );
-				entry_free( e );
-				if ( continuemode ) continue;
-				put_rc = EXIT_FAILURE;
-				break;
-			}
-		}
-
-		if ( verbose ) {
-			if ( dryrun ) {
-				fprintf( stderr, "added: \"%s\"\n",
-					e->e_dn );
-			} else {
-				fprintf( stderr, "added: \"%s\" (%08lx)\n",
-					e->e_dn, (long) id );
-			}
-		}
-
-		entry_free( e );
-
-	} while (1);
-	ldap_pvt_thread_mutex_unlock( &put_mutex1 );
-	return NULL;
-}
 
 int
 slapadd( int argc, char **argv )
 {
-	char		*buf = NULL;
-	int         lineno;
-	int         lmax;
-	int			rc = EXIT_SUCCESS;
-
+	char *buf = NULL;
 	const char *text;
 	char textbuf[SLAP_TEXT_BUFLEN] = { '\0' };
 	size_t textlen = sizeof textbuf;
+	const char *progname = "slapadd";
 
 	struct berval csn;
 	struct berval maxcsn;
-	int match;
+	struct berval bvtext;
 	Attribute *attr;
 	Entry *ctxcsn_e;
 	ID	ctxcsn_id, id;
+	OperationBuffer opbuf;
+	Operation *op;
+
+	int match;
 	int ret;
-	int i, checkvals;
-	struct berval mc;
-	ldap_pvt_thread_t put_tid;
+	int checkvals;
+	int lineno;
+	int lmax;
+	int rc = EXIT_SUCCESS;
+	int manage = 0;	
 
 	slap_tool_init( progname, SLAPADD, argc, argv );
 
+	memset( &opbuf, 0, sizeof(opbuf) );
+	op = (Operation *) &opbuf;
+
 	if( !be->be_entry_open ||
 		!be->be_entry_close ||
-		!be->be_entry_put )
+		!be->be_entry_put ||
+		(update_ctxcsn &&
+		 (!be->be_dn2id_get ||
+		  !be->be_id2entry_get ||
+		  !be->be_entry_modify)) )
 	{
 		fprintf( stderr, "%s: database doesn't support necessary operations.\n",
 			progname );
@@ -143,21 +92,6 @@ slapadd( int argc, char **argv )
 
 	checkvals = (slapMode & SLAP_TOOL_QUICK) ? 0 : 1;
 
-	if ( use_thread ) {
-		ldap_pvt_thread_initialize();
-		ldap_pvt_thread_cond_init( &put_cond1 );
-		ldap_pvt_thread_cond_init( &put_cond2 );
-		ldap_pvt_thread_mutex_init( &put_mutex1 );
-		ldap_pvt_thread_mutex_init( &put_mutex2 );
-		rc = ldap_pvt_thread_create( &put_tid, 0, do_put, NULL );
-		if ( rc ) {
-			fprintf( stderr, "%s: could not create thread.\n",
-				progname );
-			exit( EXIT_FAILURE );
-		}
-		ldap_pvt_thread_mutex_lock( &put_mutex2 );
-	}
-
 	lmax = 0;
 	lineno = 0;
 
@@ -191,7 +125,8 @@ slapadd( int argc, char **argv )
 		}
 
 		/* make sure the DN is not empty */
-		if( !e->e_nname.bv_len ) {
+		if( BER_BVISEMPTY( &e->e_nname ) &&
+			!BER_BVISEMPTY( be->be_nsuffix )) {
 			fprintf( stderr, "%s: empty dn=\"%s\" (line=%d)\n",
 				progname, e->e_dn, lineno );
 			rc = EXIT_FAILURE;
@@ -220,7 +155,7 @@ slapadd( int argc, char **argv )
 			break;
 		}
 
-		if( global_schemacheck ) {
+		{
 			Attribute *sc = attr_find( e->e_attrs,
 				slap_schema.si_ad_structuralObjectClass );
 			Attribute *oc = attr_find( e->e_attrs,
@@ -237,9 +172,9 @@ slapadd( int argc, char **argv )
 			}
 
 			if( sc == NULL ) {
-				struct berval vals[2];
+				struct berval val;
 
-				rc = structural_class( oc->a_vals, vals,
+				rc = structural_class( oc->a_vals, &val,
 					NULL, &text, textbuf, textlen );
 
 				if( rc != LDAP_SUCCESS ) {
@@ -251,28 +186,29 @@ slapadd( int argc, char **argv )
 					break;
 				}
 
-				vals[1].bv_len = 0;
-				vals[1].bv_val = NULL;
-
-				attr_merge( e, slap_schema.si_ad_structuralObjectClass,
-					vals, NULL /* FIXME */ );
+				attr_merge_one( e, slap_schema.si_ad_structuralObjectClass,
+					&val, NULL );
 			}
 
 			/* check schema */
-			rc = entry_schema_check( be, e, NULL, &text, textbuf, textlen );
+			op->o_bd = be;
 
-			if( rc != LDAP_SUCCESS ) {
-				fprintf( stderr, "%s: dn=\"%s\" (line=%d): (%d) %s\n",
-					progname, e->e_dn, lineno, rc, text );
-				rc = EXIT_FAILURE;
-				entry_free( e );
-				if( continuemode ) continue;
-				break;
+			if ( (slapMode & SLAP_TOOL_NO_SCHEMA_CHECK) == 0) {
+				rc = entry_schema_check( op, e, NULL, manage,
+					&text, textbuf, textlen );
+
+				if( rc != LDAP_SUCCESS ) {
+					fprintf( stderr, "%s: dn=\"%s\" (line=%d): (%d) %s\n",
+						progname, e->e_dn, lineno, rc, text );
+					rc = EXIT_FAILURE;
+					entry_free( e );
+					if( continuemode ) continue;
+					break;
+				}
 			}
 		}
 
 		if ( SLAP_LASTMOD(be) ) {
-			struct tm *ltm;
 			time_t now = slap_get_time();
 			char uuidbuf[ LDAP_LUTIL_UUIDSTR_BUFSIZE ];
 			struct berval vals[ 2 ];
@@ -289,14 +225,13 @@ slapadd( int argc, char **argv )
 			nvals[1].bv_len = 0;
 			nvals[1].bv_val = NULL;
 
-			ltm = gmtime(&now);
-			lutil_gentime( timebuf, sizeof(timebuf), ltm );
-
 			csn.bv_len = lutil_csnstr( csnbuf, sizeof( csnbuf ), 0, 0 );
 			csn.bv_val = csnbuf;
 
 			timestamp.bv_val = timebuf;
-			timestamp.bv_len = strlen(timebuf);
+			timestamp.bv_len = sizeof(timebuf);
+
+			slap_timestamp( &now, &timestamp );
 
 			if ( BER_BVISEMPTY( &be->be_rootndn ) ) {
 				BER_BVSTR( &name, SLAPD_ANONYMOUS );
@@ -311,8 +246,7 @@ slapadd( int argc, char **argv )
 			{
 				vals[0].bv_len = lutil_uuidstr( uuidbuf, sizeof( uuidbuf ) );
 				vals[0].bv_val = uuidbuf;
-				attr_merge_normalize_one( e,
-							slap_schema.si_ad_entryUUID, vals, NULL );
+				attr_merge_normalize_one( e, slap_schema.si_ad_entryUUID, vals, NULL );
 			}
 
 			if( attr_find( e->e_attrs, slap_schema.si_ad_creatorsName )
@@ -370,22 +304,6 @@ slapadd( int argc, char **argv )
 			}
 		}
 
-		if ( use_thread ) {
-			ldap_pvt_thread_mutex_lock( &put_mutex1 );
-			if (put_rc) {
-				rc = put_rc;
-				ldap_pvt_thread_mutex_unlock( &put_mutex1 );
-				break;
-			}
-			put_e = e;
-			put_lineno = lineno;
-			ldap_pvt_thread_cond_signal( &put_cond1 );
-			ldap_pvt_thread_mutex_unlock( &put_mutex1 );
-			/* Make sure writer wakes up */
-			ldap_pvt_thread_cond_wait( &put_cond2, &put_mutex2 );
-			continue;
-		}
-
 		if ( !dryrun ) {
 			id = be->be_entry_put( be, e, &bvtext );
 			if( id == NOID ) {
@@ -397,37 +315,18 @@ slapadd( int argc, char **argv )
 				if( continuemode ) continue;
 				break;
 			}
-		}
-
-		if ( verbose ) {
-			if ( dryrun ) {
-				fprintf( stderr, "added: \"%s\"\n",
-					e->e_dn );
-			} else {
+			if ( verbose )
 				fprintf( stderr, "added: \"%s\" (%08lx)\n",
 					e->e_dn, (long) id );
-			}
+		} else {
+			if ( verbose )
+				fprintf( stderr, "added: \"%s\"\n",
+					e->e_dn );
 		}
 
 		entry_free( e );
 	}
 
-	if ( use_thread ) {
-		ldap_pvt_thread_mutex_unlock( &put_mutex2 );
-		ldap_pvt_thread_mutex_lock( &put_mutex1 );
-		/* Tell child thread to stop if it hasn't aborted */
-		if ( !put_rc ) {
-			put_rc = EXIT_FAILURE;
-			ldap_pvt_thread_cond_signal( &put_cond1 );
-		}
-		ldap_pvt_thread_mutex_unlock( &put_mutex1 );
-		ldap_pvt_thread_join( put_tid, NULL );
-		ldap_pvt_thread_mutex_destroy( &put_mutex2 );
-		ldap_pvt_thread_mutex_destroy( &put_mutex1 );
-		ldap_pvt_thread_cond_destroy( &put_cond2 );
-		ldap_pvt_thread_cond_destroy( &put_cond1 );
-	}
-
 	bvtext.bv_len = textlen;
 	bvtext.bv_val = textbuf;
 	bvtext.bv_val[0] = '\0';
@@ -442,15 +341,21 @@ slapadd( int argc, char **argv )
 			if ( ret == LDAP_SUCCESS ) {
 				attr = attr_find( ctxcsn_e->e_attrs,
 									slap_schema.si_ad_contextCSN );
-				value_match( &match, slap_schema.si_ad_entryCSN,
-					slap_schema.si_ad_entryCSN->ad_type->sat_ordering,
-					SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX,
-					&maxcsn, &attr->a_nvals[0], &text );
+				if ( attr ) {
+					value_match( &match, slap_schema.si_ad_entryCSN,
+						slap_schema.si_ad_entryCSN->ad_type->sat_ordering,
+						SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX,
+						&maxcsn, &attr->a_nvals[0], &text );
+					if ( match > 0 ) {
+						AC_MEMCPY( attr->a_vals[0].bv_val, maxcsn.bv_val, maxcsn.bv_len );
+						attr->a_vals[0].bv_val[maxcsn.bv_len] = '\0';
+						attr->a_vals[0].bv_len = maxcsn.bv_len;
+					}
+				} else {
+					match = 1;
+					attr_merge_one( ctxcsn_e, slap_schema.si_ad_contextCSN, &maxcsn, NULL );
+				}
 				if ( match > 0 ) {
-					AC_MEMCPY( attr->a_vals[0].bv_val, maxcsn.bv_val, maxcsn.bv_len );
-					attr->a_vals[0].bv_val[maxcsn.bv_len] = '\0';
-					attr->a_vals[0].bv_len = maxcsn.bv_len;
-				
 					ctxcsn_id = be->be_entry_modify( be, ctxcsn_e, &bvtext );
 					if( ctxcsn_id == NOID ) {
 						fprintf( stderr, "%s: could not modify ctxcsn\n",
