@@ -1,5 +1,5 @@
 /* slapi_overlay.c - SLAPI overlay */
-/* $OpenLDAP: pkg/ldap/servers/slapd/slapi/slapi_overlay.c,v 1.40 2006/02/14 08:20:50 lukeh Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/slapi/slapi_overlay.c,v 1.40.2.1 2006/05/15 17:04:44 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2001-2006 The OpenLDAP Foundation.
@@ -230,7 +230,7 @@ slapi_over_result( Operation *op, SlapReply *rs, int type )
 {
 	Slapi_PBlock		*pb = SLAPI_OPERATION_PBLOCK( op );
 
-	assert( rs->sr_type == REP_RESULT );
+	assert( rs->sr_type == REP_RESULT || rs->sr_type == REP_SASL || rs->sr_type == REP_EXTENDED );
 
 	slapi_over_call_plugins( pb, type );
 
@@ -499,6 +499,8 @@ slapi_over_response( Operation *op, SlapReply *rs )
 	if ( pb->pb_intop == 0 ) {
 		switch ( rs->sr_type ) {
 		case REP_RESULT:
+		case REP_SASL:
+		case REP_EXTENDED:
 			rc = slapi_over_result( op, rs, SLAPI_PLUGIN_PRE_RESULT_FN );
 			break;
 		case REP_SEARCH:
@@ -528,6 +530,8 @@ slapi_over_cleanup( Operation *op, SlapReply *rs )
 	if ( pb->pb_intop == 0 ) {
 		switch ( rs->sr_type ) {
 		case REP_RESULT:
+		case REP_SASL:
+		case REP_EXTENDED:
 			rc = slapi_over_result( op, rs, SLAPI_PLUGIN_POST_RESULT_FN );
 			break;
 		case REP_SEARCH:
