@@ -1,8 +1,8 @@
 /* syntax.c - routines to manage syntax definitions */
-/* $OpenLDAP: pkg/ldap/servers/slapd/syntax.c,v 1.40 2004/09/26 20:58:48 ando Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/syntax.c,v 1.43 2006/01/03 22:12:16 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -178,15 +178,16 @@ register_syntax(
 
 	code = syn_add( syn, def, &err );
 
-	ldap_memfree( syn );
-
 	if ( code ) {
 		Debug( LDAP_DEBUG_ANY, "Error in register_syntax: %s %s in %s\n",
 		    scherr2str(code), err, def->sd_desc );
+		ldap_syntax_free( syn );
 
 		return( -1 );
 	}
 
+	ldap_memfree( syn );
+
 	return( 0 );
 }
 
