@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/tests/progs/slapd-tester.c,v 1.46.2.1 2006/04/11 16:07:20 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/tests/progs/slapd-tester.c,v 1.46.2.2 2006/05/15 17:04:45 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1999-2006 The OpenLDAP Foundation.
@@ -162,6 +162,7 @@ main( int argc, char **argv )
 	char		*bfile = NULL;
 	char		*breqs[MAXREQS];
 	char		*bcreds[MAXREQS];
+	char		*battrs[MAXREQS];
 	int		bnum = 0;
 	char		*bargs[MAXARGS];
 	int		banum;
@@ -309,7 +310,7 @@ main( int argc, char **argv )
 
 	/* look for bind requests */
 	if ( bfile ) {
-		bnum = get_search_filters( bfile, bcreds, NULL, breqs );
+		bnum = get_search_filters( bfile, bcreds, battrs, breqs );
 	}
 
 	/* setup friendly option */
@@ -642,6 +643,13 @@ main( int argc, char **argv )
 
 			bargs[banum - 4] = breqs[jj];
 			bargs[banum - 2] = bcreds[jj];
+			if ( battrs[jj] != NULL ) {
+				bargs[banum - 1] = "-a";
+				bargs[banum] = battrs[jj];
+
+			} else {
+				sargs[sanum - 1] = NULL;
+			}
 			fork_child( bcmd, bargs );
 		}
 	}
